Package: BivLaplaceRL
Type: Package
Title: Bivariate Laplace Transforms, Stochastic Orders, and Entropy
        Measures in Reliability
Version: 1.0.0
Authors@R: c(
    person("Mahesh", "Divakaran",
           email = "imaheshdivakaran@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3488-0857",
                       affiliation = "Amity School of Applied Sciences, Amity University Lucknow")),
    person("S.", "Jayalekshmi",
           role = c("aut", "ctb"),
           comment = c(affiliation = "Department of Statistics, Cochin University of Science and Technology")),
    person("G.", "Rajesh",
           email = "rajeshgstat@gmail.com",
           role = c("aut"),
           comment = c(affiliation = "Department of Statistics, Cochin University of Science and Technology")),
    person("N. Unnikrishnan", "Nair",
           role = c("aut"),
           comment = c(affiliation = "Department of Statistics, Cochin University of Science and Technology"))
  )
Description: Implements methods for bivariate and univariate Laplace transforms
    of residual lives and reversed residual lives, associated stochastic ordering
    concepts, and entropy measures for reliability analysis. The package covers:
    (1) Bivariate Laplace transform of residual lives and stochastic comparisons
    based on the bivariate Laplace transform order of residual lives (BLt-rl),
    including weak bivariate hazard rate, mean residual life, and relative mean
    residual life orders, nonparametric estimation, and NBUHR/NWUHR aging class
    characterisation; Jayalekshmi, Rajesh, and Nair (2022)
    "Bivariate Laplace Transform of Residual Lives and Their Properties"
    <doi:10.1080/03610926.2022.2085874>;
    (2) Bivariate Laplace transform order of reversed residual lives (BLt-Rrl),
    reversed hazard gradient, reversed mean residual life, and the associated
    stochastic orders (weak bivariate reversed hazard rate, weak bivariate
    reversed mean residual life); Jayalekshmi, Rajesh, and Nair (2022)
    "Bivariate Laplace Transform Order and Ordering of Reversed Residual Lives"
    <doi:10.1142/S0218539322500061>;
    (3) Univariate Laplace transform of residual life, hazard rate, mean residual
    life, and the corresponding stochastic orders (Lt-rl order, hazard rate order,
    MRL order), together with a nonparametric estimator.
    Shannon entropy and Golomb's (1966) information generating function are also
    provided. Parametric families supported include the Gumbel bivariate
    exponential, Farlie-Gumbel-Morgenstern (FGM), bivariate power, and
    Schur-constant distributions. Plotting utilities and a simulation framework
    for evaluating estimator performance are also provided.
License: GPL-3
URL: https://itsmdivakaran.github.io/BivLaplaceRL/,
        https://github.com/itsmdivakaran/BivLaplaceRL
BugReports: https://github.com/itsmdivakaran/BivLaplaceRL/issues
Encoding: UTF-8
Config/Needs/website: pkgdown
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: stats, graphics
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, MASS, survival, covr
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Language: en-GB
Packaged: 2026-04-30 06:21:52 UTC; maheshdivakaran
Author: Mahesh Divakaran [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3488-0857>, affiliation: Amity School
    of Applied Sciences, Amity University Lucknow),
  S. Jayalekshmi [aut, ctb] (affiliation: Department of Statistics,
    Cochin University of Science and Technology),
  G. Rajesh [aut] (affiliation: Department of Statistics, Cochin
    University of Science and Technology),
  N. Unnikrishnan Nair [aut] (affiliation: Department of Statistics,
    Cochin University of Science and Technology)
Maintainer: Mahesh Divakaran <imaheshdivakaran@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-30 10:40:02 UTC
Built: R 4.6.0; ; 2026-04-30 11:22:27 UTC; unix

# Generated by roxygen2: do not edit by hand

export(biv_brlmr_order)
export(biv_hazard_gradient)
export(biv_mean_residual)
export(biv_rhazard_gradient)
export(biv_rmrl)
export(biv_whr_order)
export(biv_wmrl_order)
export(biv_wrhr_order)
export(biv_wrmrl_order)
export(blt_order_residual)
export(blt_order_reversed)
export(blt_residual)
export(blt_residual_gumbel)
export(blt_reversed)
export(blt_reversed_fgm)
export(blt_reversed_power)
export(dbivpower)
export(dfgm_biv)
export(dgumbel_biv)
export(hazard_rate)
export(hr_order)
export(info_gen_function)
export(lt_residual)
export(lt_rl_order)
export(mean_residual)
export(mrl_order)
export(nbuhr_test)
export(np_blt_residual)
export(np_lt_residual)
export(pbivpower)
export(pfgm_biv)
export(pgumbel_biv)
export(plot_blt_residual)
export(plot_blt_reversed)
export(rbivpower)
export(rfgm_biv)
export(rgumbel_biv)
export(rschur_biv)
export(sbivpower)
export(sfgm_biv)
export(sgumbel_biv)
export(shannon_entropy)
export(sim_blt_residual)
export(sschur_biv)

# BivLaplaceRL <img src="man/figures/logo.png" align="right" height="139" alt="" />

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/BivLaplaceRL)](https://CRAN.R-project.org/package=BivLaplaceRL)
[![R-CMD-check](https://github.com/itsmdivakaran/BivLaplaceRL/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/itsmdivakaran/BivLaplaceRL/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/itsmdivakaran/BivLaplaceRL/branch/main/graph/badge.svg)](https://app.codecov.io/gh/itsmdivakaran/BivLaplaceRL?branch=main)
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

**BivLaplaceRL** is an R package for bivariate and univariate Laplace transforms
of residual lives, stochastic ordering concepts, and entropy measures in
reliability analysis.

<div class="feature-row">

<div class="feature-card">
<h4>Residual Life Analysis</h4>
<p>Bivariate Laplace transforms of residual lives — closed-form Gumbel results,
general numerical integration, nonparametric estimation, and NBUHR/NWUHR aging
class characterisation.</p>
</div>

<div class="feature-card">
<h4>Reversed Residual Lives</h4>
<p>BLt-Rrl framework: reversed hazard gradient, reversed mean residual life, and
closed-form transforms for FGM and bivariate power distributions.</p>
</div>

<div class="feature-card">
<h4>Univariate Methods</h4>
<p>Univariate LT of residual life, hazard rate, mean residual life, and three
stochastic orders (Lt-rl, hazard rate, MRL) with nonparametric estimation.</p>
</div>

<div class="feature-card">
<h4>Stochastic Orders</h4>
<p>Seven bivariate and three univariate stochastic order checks, each returning
a logical flag and supporting diagnostic values.</p>
</div>

</div>

## Research Basis

| Paper | Journal | Authors |
|-------|---------|---------|
| Bivariate Laplace transform of residual lives and their properties | *Communications in Statistics — Theory and Methods* (2022) | Jayalekshmi S., Rajesh G., Nair N.U. |
| Bivariate Laplace transform order and ordering of reversed residual lives | *Int. J. Reliability, Quality and Safety Engineering* | Jayalekshmi S., Rajesh G. |

## Features

### Parametric Distributions
- Gumbel bivariate exponential (`dgumbel_biv`, `sgumbel_biv`, `rgumbel_biv`, `pgumbel_biv`)
- Farlie-Gumbel-Morgenstern — FGM (`dfgm_biv`, `pfgm_biv`, `sfgm_biv`, `rfgm_biv`)
- Bivariate power distribution (`dbivpower`, `pbivpower`, `sbivpower`, `rbivpower`)
- Schur-constant distribution (`sschur_biv`, `rschur_biv`)

### Bivariate Laplace Transform of Residual Lives
- `blt_residual()` — numerical computation for any survival function
- `blt_residual_gumbel()` — closed-form for Gumbel distribution
- `biv_hazard_gradient()` — bivariate hazard gradient
- `biv_mean_residual()` — bivariate mean residual life
- `nbuhr_test()` — NBUHR/NWUHR aging class test
- `np_blt_residual()` — nonparametric estimator
- `sim_blt_residual()` — Monte-Carlo simulation study

### Bivariate Laplace Transform of Reversed Residual Lives
- `blt_reversed()` — for any joint CDF
- `blt_reversed_fgm()` — closed form for FGM
- `blt_reversed_power()` — for bivariate power distribution
- `biv_rhazard_gradient()` — reversed hazard gradient
- `biv_rmrl()` — reversed mean residual life

### Univariate Residual Life Analysis
- `lt_residual()` — LT of residual life: E[e^{-sX} | X > t]
- `hazard_rate()` — hazard rate h(t) = f(t)/S(t)
- `mean_residual()` — mean residual life m(t) = E[X-t | X>t]
- `np_lt_residual()` — nonparametric estimator

### Stochastic Orders (Bivariate)
- `blt_order_residual()` — BLt-rl order
- `blt_order_reversed()` — BLt-Rrl order
- `biv_whr_order()` — weak bivariate hazard rate order
- `biv_wmrl_order()` — weak bivariate MRL order
- `biv_brlmr_order()` — bivariate relative MRL order
- `biv_wrhr_order()` — weak bivariate reversed hazard rate order
- `biv_wrmrl_order()` — weak bivariate reversed MRL order

### Stochastic Orders (Univariate)
- `lt_rl_order()` — Lt-rl order: L_X(s,t) ≤ L_Y(s,t) for all s, t
- `hr_order()` — hazard rate order: h_X(t) ≤ h_Y(t) for all t
- `mrl_order()` — MRL order: m_X(t) ≤ m_Y(t) for all t

### Entropy Measures
- `shannon_entropy()` — Shannon differential entropy
- `info_gen_function()` — Golomb information generating function

### Plotting
- `plot_blt_residual()`, `plot_blt_reversed()`

## Installation

```r
# Install from CRAN
install.packages("BivLaplaceRL")
```

```r
# Development version from GitHub
# install.packages("devtools")
devtools::install_github("itsmdivakaran/BivLaplaceRL")
```

## Quick Start

```r
library(BivLaplaceRL)

# 1. Simulate from Gumbel bivariate exponential
set.seed(42)
dat <- rgumbel_biv(500, k1 = 1, k2 = 1, theta = 0.5)

# 2. Nonparametric estimate of BLT of residual lives
np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3)

# 3. Compare with closed-form
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, k1 = 1, k2 = 1, theta = 0.5)

# 4. Univariate LT of residual life for Exp(1)
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
lt_residual(f, Fb, s = 1, t = 0.5)

# 5. Hazard rate and MRL
hazard_rate(f, Fb, t = c(0.5, 1, 2))
mean_residual(Fb, t = c(0, 0.5, 1, 2))

# 6. Check univariate stochastic orders: Exp(2) <=_hr Exp(1)?
f2  <- function(x) dexp(x, 2)
Fb2 <- function(x) pexp(x, 2, lower.tail = FALSE)
hr_order(f2, Fb2, f, Fb, t_grid = c(0.5, 1, 2))$order_holds
```

## Authors

**Mahesh Divakaran** (maintainer)
Research Scholar, Amity School of Applied Sciences, Amity University Lucknow
imaheshdivakaran@gmail.com

**S. Jayalekshmi**, **G. Rajesh**, **N. Unnikrishnan Nair**
Department of Statistics, Cochin University of Science and Technology

## References

Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform of
residual lives and their properties. *Communications in Statistics — Theory and
Methods*. <https://doi.org/10.1080/03610926.2022.2085874>

Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*. <https://doi.org/10.1142/S0218539322500061>

Belzunce F., Ortega E., Ruiz J.M. (1999). The Laplace order and ordering of
residual lives. *Statistics & Probability Letters*, 42(2), 145--156. 
<https://doi.org/10.1016/S0167-7152(98)00202-8>

## License

GPL-3 © 2024 Mahesh Divakaran

#' BivLaplaceRL: Bivariate Laplace Transforms, Stochastic Orders, and Entropy
#' Measures in Reliability
#'
#' @description
#' \pkg{BivLaplaceRL} provides a unified framework for three interconnected
#' research themes in reliability analysis and survival theory:
#'
#' \strong{1. Bivariate Laplace Transform of Residual Lives}
#'
#' Implements the vector Laplace transform of bivariate residual lives
#' \eqn{(L_{X_{t_1|t_2}}(s_1),\,L_{X_{t_2|t_1}}(s_2))}, the associated
#' stochastic ordering BLt-rl, and its relationships with the weak bivariate
#' hazard rate order, weak bivariate mean residual life order, and bivariate
#' relative mean residual life order.  Nonparametric estimation and NBUHR/NWUHR
#' aging class tests are included.
#' Reference: Jayalekshmi, Rajesh, and Nair (2022)
#' \doi{10.1080/03610926.2022.2085874}.
#'
#' \strong{2. Bivariate Laplace Transform of Reversed Residual Lives}
#'
#' Implements the bivariate Laplace transform of reversed (past) residual lives,
#' the BLt-Rrl stochastic order, and connections with weak bivariate reversed
#' hazard rate and reversed mean residual life orders.
#'
#' \strong{3. Residual Entropy Generating Function}
#'
#' Implements the dynamic information generating function (REGF), the residual
#' entropy of Ebrahimi and Pellerey (1995), Golomb's (1966) information
#' generating function, and nonparametric estimation of REGF with Monte-Carlo
#' simulation support.
#'
#' @section Parametric families:
#' Gumbel bivariate exponential, Farlie-Gumbel-Morgenstern (FGM), bivariate
#' power, Schur-constant, and exponential distributions are built in.
#'
#' @section Plotting:
#' \code{\link{plot_blt_residual}}, \code{\link{plot_blt_reversed}}, and
#' \code{\link{plot_regf}} provide ready-made visualisations.
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform of
#' residual lives and their properties. *Communications in Statistics - Theory
#' and Methods*. \doi{10.1080/03610926.2022.2085874}
#'
#' Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
#' reversed residual lives. *International Journal of Reliability, Quality and
#' Safety Engineering*.
#'
#' Smitha S., Rajesh G., Jayalekshmi S. (2024). On residual entropy generating
#' function. *Journal of the Indian Statistical Association*, 62(1), 81--93.
#'
#' Belzunce F., Ortega E., Ruiz J.M. (1999). The Laplace order and ordering of
#' residual lives. *Statistics & Probability Letters*, 42(2), 145--156.
#'
#' Ebrahimi N., Pellerey F. (1995). New partial ordering of survival functions
#' based on the notion of uncertainty. *Journal of Applied Probability*, 32(1),
#' 202--211.
#'
#' Golomb S.W. (1966). The information generating function of a probability
#' distribution. *IEEE Transactions on Information Theory*, 12(1), 75--77.
#'
#' @docType package
#' @name BivLaplaceRL-package
#' @aliases BivLaplaceRL
"_PACKAGE"

# ==============================================================================
# Parametric Bivariate Distributions
# ==============================================================================

# ------------------------------------------------------------------------------
# Gumbel Bivariate Exponential
# ------------------------------------------------------------------------------

#' Gumbel Bivariate Exponential Distribution
#'
#' @description
#' Density, distribution (survival) function, and random generation for the
#' Gumbel bivariate exponential distribution with parameters \eqn{k_1},
#' \eqn{k_2}, and association parameter \eqn{\theta}.
#'
#' The joint survival function is
#' \deqn{\bar{F}(x_1,x_2) = \exp(-k_1 x_1 - k_2 x_2 - \theta x_1 x_2),
#'   \quad x_1,x_2 > 0,\; k_1,k_2 > 0,\; 0 \le \theta \le k_1 k_2.}
#'
#' @param x1,x2 Non-negative numeric values or vectors.
#' @param k1,k2 Positive rate parameters.
#' @param theta Non-negative association parameter; must satisfy
#'   \eqn{0 \le \theta \le k_1 k_2}.
#' @param n Number of random observations.
#' @param log.p Logical; if \code{TRUE} probabilities are given as
#'   \eqn{\log(p)}.
#'
#' @return
#' \describe{
#'   \item{\code{dgumbel_biv}}{Numeric vector of density values.}
#'   \item{\code{pgumbel_biv}}{Numeric vector of joint CDF values.}
#'   \item{\code{sgumbel_biv}}{Numeric vector of joint survival function values.}
#'   \item{\code{rgumbel_biv}}{A two-column matrix with columns \code{X1} and
#'     \code{X2} containing the simulated observations.}
#' }
#'
#' @examples
#' # Survival function
#' sgumbel_biv(1, 2, k1 = 1, k2 = 1, theta = 0.5)
#'
#' # Density
#' dgumbel_biv(0.5, 0.5, k1 = 1, k2 = 1.5, theta = 0.3)
#'
#' # Random sample
#' set.seed(42)
#' dat <- rgumbel_biv(100, k1 = 1, k2 = 1, theta = 0.5)
#' head(dat)
#'
#' @references
#' Gumbel E.J. (1960). Bivariate exponential distributions. *Journal of the
#' American Statistical Association*, 55(292), 698--707.
#'
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @name gumbel_biv
NULL

#' @rdname gumbel_biv
#' @export
dgumbel_biv <- function(x1, x2, k1 = 1, k2 = 1, theta = 0, log.p = FALSE) {
  .check_positive(k1, "k1"); .check_positive(k2, "k2")
  .check_nonneg(theta, "theta")
  if (theta > k1 * k2)
    stop("`theta` must be <= k1 * k2 = ", k1 * k2, ".")
  if (any(x1 < 0) || any(x2 < 0))
    stop("`x1` and `x2` must be non-negative.")

  log_d <- log(k1 + theta * x2) + log(k2 + theta * x1) +
    (-k1 * x1 - k2 * x2 - theta * x1 * x2) -
    log(1 + theta * x1 * x2 / ((k1 + theta * x2) * (k2 + theta * x1)))
  # Exact density: d/dx1 d/dx2 F(x1,x2) via survival function
  # f(x1,x2) = (k1 + theta*x2)(k2 + theta*x1) * exp(-k1*x1 - k2*x2 - theta*x1*x2)
  # minus theta * exp(-k1*x1 - k2*x2 - theta*x1*x2)  [cross term]
  log_d2 <- log(pmax(
    (k1 + theta * x2) * (k2 + theta * x1) - theta,
    .Machine$double.eps
  )) + (-k1 * x1 - k2 * x2 - theta * x1 * x2)

  if (log.p) return(log_d2)
  exp(log_d2)
}

#' @rdname gumbel_biv
#' @export
sgumbel_biv <- function(x1, x2, k1 = 1, k2 = 1, theta = 0, log.p = FALSE) {
  .check_positive(k1, "k1"); .check_positive(k2, "k2")
  .check_nonneg(theta, "theta")
  if (theta > k1 * k2)
    stop("`theta` must be <= k1 * k2 = ", k1 * k2, ".")
  log_s <- -k1 * x1 - k2 * x2 - theta * x1 * x2
  if (log.p) return(log_s)
  exp(log_s)
}

#' @rdname gumbel_biv
#' @export
pgumbel_biv <- function(x1, x2, k1 = 1, k2 = 1, theta = 0) {
  # P(X1 <= x1, X2 <= x2) via inclusion-exclusion on the survival function
  # F(x1,x2) = 1 - S(0,x2) - S(x1,0) + S(x1,x2)
  # = 1 - e^{-k2*x2} - e^{-k1*x1} + e^{-k1*x1-k2*x2-theta*x1*x2}
  1 - exp(-k1 * x1) - exp(-k2 * x2) +
    sgumbel_biv(x1, x2, k1, k2, theta)
}

#' @rdname gumbel_biv
#' @export
rgumbel_biv <- function(n, k1 = 1, k2 = 1, theta = 0) {
  .check_positive(k1, "k1"); .check_positive(k2, "k2")
  .check_nonneg(theta, "theta")
  # Conditional simulation: X1 ~ Exp(k1), X2|X1=x1 ~ Exp(k2 + theta*x1)
  x1 <- stats::rexp(n, rate = k1)
  x2 <- stats::rexp(n, rate = k2 + theta * x1)
  matrix(c(x1, x2), ncol = 2L, dimnames = list(NULL, c("X1", "X2")))
}

# ------------------------------------------------------------------------------
# Farlie-Gumbel-Morgenstern (FGM) Distribution
# ------------------------------------------------------------------------------

#' Farlie-Gumbel-Morgenstern (FGM) Bivariate Distribution
#'
#' @description
#' Density, distribution function, survival function, and random generation for
#' the FGM bivariate distribution on \eqn{[0,1]^2}:
#' \deqn{F(x_1, x_2) = x_1 x_2 \bigl[1 + \theta(1-x_1)(1-x_2)\bigr],
#'   \quad 0 \le x_1, x_2 \le 1,\; |\theta| \le 1.}
#'
#' @param x1,x2 Values in \eqn{[0,1]}.
#' @param theta Association parameter, \eqn{|\theta| \le 1}.
#' @param n Number of random observations.
#'
#' @return Numeric vector (scalar functions) or two-column matrix
#'   (\code{rfgm_biv}).
#'
#' @examples
#' pfgm_biv(0.4, 0.6, theta = 0.5)
#' dfgm_biv(0.4, 0.6, theta = 0.5)
#' set.seed(1); head(rfgm_biv(50, theta = 0.5))
#'
#' @references
#' Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
#' reversed residual lives. *International Journal of Reliability, Quality and
#' Safety Engineering*.
#'
#' @name fgm_biv
NULL

#' @rdname fgm_biv
#' @export
pfgm_biv <- function(x1, x2, theta = 0) {
  .check_range(theta, -1, 1, "theta")
  x1 * x2 * (1 + theta * (1 - x1) * (1 - x2))
}

#' @rdname fgm_biv
#' @export
dfgm_biv <- function(x1, x2, theta = 0) {
  .check_range(theta, -1, 1, "theta")
  1 + theta * (1 - 2 * x1) * (1 - 2 * x2)
}

#' @rdname fgm_biv
#' @export
sfgm_biv <- function(x1, x2, theta = 0) {
  # S(x1,x2) = 1 - x1 - x2 + F(x1,x2)
  1 - x1 - x2 + pfgm_biv(x1, x2, theta)
}

#' @rdname fgm_biv
#' @export
rfgm_biv <- function(n, theta = 0) {
  .check_range(theta, -1, 1, "theta")
  # Conditional distribution: X1 ~ Uniform(0,1)
  # X2|X1=u: solve F_{2|1}(x2|u) = v
  u  <- stats::runif(n)
  v  <- stats::runif(n)
  a  <- 1 + theta * (1 - 2 * u)
  b  <- -1
  cc <- v
  # a*x2^2 + b*x2 + cc = 0 => only root in [0,1]
  disc <- pmax(b^2 - 4 * a * cc, 0)
  x2 <- (-b - sqrt(disc)) / (2 * a)
  x2 <- pmin(pmax(x2, 0), 1)
  matrix(c(u, x2), ncol = 2L, dimnames = list(NULL, c("X1", "X2")))
}

# ------------------------------------------------------------------------------
# Bivariate Power Distribution
# ------------------------------------------------------------------------------

#' Bivariate Power Distribution
#'
#' @description
#' Distribution function, survival function, density, and random generation for
#' the bivariate power distribution:
#' \deqn{F(x_1,x_2) = x_1^{p_1 + \theta \log x_2}\,
#'   x_2^{p_2},\quad 0 \le x_1,x_2 \le p_2,\; p_1,p_2 > 0,\; 0 \le \theta \le 1.}
#'
#' @param x1,x2 Values in the support.
#' @param p1,p2 Positive shape parameters.
#' @param theta Association parameter, \eqn{0 \le \theta \le 1}.
#' @param n Number of random observations.
#'
#' @return Numeric vector or two-column matrix (\code{rbivpower}).
#'
#' @examples
#' pbivpower(0.4, 0.5, p1 = 2, p2 = 2, theta = 0.3)
#' set.seed(7); head(rbivpower(30, p1 = 2, p2 = 2, theta = 0.2))
#'
#' @references
#' Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
#' reversed residual lives. *International Journal of Reliability, Quality and
#' Safety Engineering*.
#'
#' @name bivpower
NULL

#' @rdname bivpower
#' @export
pbivpower <- function(x1, x2, p1 = 1, p2 = 1, theta = 0) {
  .check_positive(p1, "p1"); .check_positive(p2, "p2")
  .check_range(theta, 0, 1, "theta")
  x1^(p1 + theta * log(pmax(x2, .Machine$double.eps))) *
    x2^p2
}

#' @rdname bivpower
#' @export
sbivpower <- function(x1, x2, p1 = 1, p2 = 1, theta = 0) {
  # Survival function via inclusion-exclusion (marginals are power/uniform)
  # F_1(x1) = x1^p1, F_2(x2) = x2^p2 on [0,1]
  1 - x1^p1 - x2^p2 + pbivpower(x1, x2, p1, p2, theta)
}

#' @rdname bivpower
#' @export
dbivpower <- function(x1, x2, p1 = 1, p2 = 1, theta = 0) {
  .check_positive(p1, "p1"); .check_positive(p2, "p2")
  # d/dx1 d/dx2 F(x1,x2)
  alpha <- p1 + theta * log(pmax(x2, .Machine$double.eps))
  dbeta <- theta / pmax(x2, .Machine$double.eps)
  term1 <- alpha * (p2 + dbeta * x1 * log(pmax(x1, .Machine$double.eps))) *
    x1^(alpha - 1) * x2^(p2 - 1)
  term2 <- alpha * x1^(alpha - 1) * x2^p2 * dbeta
  term1 - term2
}

#' @rdname bivpower
#' @export
rbivpower <- function(n, p1 = 1, p2 = 1, theta = 0) {
  # Simple accept-reject or conditional simulation
  x2 <- stats::runif(n)^(1 / p2)
  alpha <- p1 + theta * log(pmax(x2, .Machine$double.eps))
  x1 <- stats::runif(n)^(1 / pmax(alpha, .Machine$double.eps))
  matrix(c(x1, x2), ncol = 2L, dimnames = list(NULL, c("X1", "X2")))
}

# ------------------------------------------------------------------------------
# Schur-Constant Distribution
# ------------------------------------------------------------------------------

#' Schur-Constant Bivariate Distribution
#'
#' @description
#' Random generation and survival function for a Schur-constant bivariate
#' distribution with survival function
#' \deqn{\bar{F}(x_1,x_2) = S(x_1 + x_2),\quad x_1,x_2 > 0,}
#' where \eqn{S} is a given univariate survival function.  The default marginal
#' is exponential with rate \code{lambda}.
#'
#' @param x1,x2 Non-negative values.
#' @param lambda Exponential rate parameter for the generating survival function.
#' @param n Number of random observations.
#'
#' @return Numeric vector (\code{sschur_biv}) or two-column matrix
#'   (\code{rschur_biv}).
#'
#' @examples
#' sschur_biv(0.5, 1, lambda = 1)
#' set.seed(2); head(rschur_biv(40, lambda = 1))
#'
#' @references
#' Barlow R.E., Mendel M.B. (1992). De Finetti-type representations for life
#' distributions. *Journal of the American Statistical Association*, 87(420),
#' 1116--1122.
#'
#' @name schur_biv
NULL

#' @rdname schur_biv
#' @export
sschur_biv <- function(x1, x2, lambda = 1) {
  .check_positive(lambda, "lambda")
  exp(-lambda * (x1 + x2))
}

#' @rdname schur_biv
#' @export
rschur_biv <- function(n, lambda = 1) {
  .check_positive(lambda, "lambda")
  # Dirichlet-like construction: (X1,X2) = (U*T, (1-U)*T) where T ~ Exp(lambda)
  T_  <- stats::rexp(n, rate = lambda)
  U   <- stats::runif(n)
  matrix(c(U * T_, (1 - U) * T_), ncol = 2L,
         dimnames = list(NULL, c("X1", "X2")))
}

test_that("sgumbel_biv returns 1 at origin", {
  expect_equal(sgumbel_biv(0, 0, k1 = 1, k2 = 1, theta = 0.5), 1)
})

test_that("sgumbel_biv is non-negative and <= 1", {
  vals <- outer(seq(0, 3, 0.5), seq(0, 3, 0.5),
                function(x, y) sgumbel_biv(x, y, 1, 1, 0.3))
  expect_true(all(vals >= 0 & vals <= 1))
})

test_that("sgumbel_biv decreases in both arguments", {
  s1 <- sgumbel_biv(0.5, 1, 1, 1, 0.3)
  s2 <- sgumbel_biv(1.0, 1, 1, 1, 0.3)
  s3 <- sgumbel_biv(0.5, 2, 1, 1, 0.3)
  expect_gt(s1, s2)
  expect_gt(s1, s3)
})

test_that("dgumbel_biv integrates to approx 1 (independence case)", {
  # Double-integrate using nested stats::integrate (no extra packages needed).
  # The outer integrand must be Vectorized so stats::integrate can pass vectors.
  inner <- function(x2, x1_val)
    stats::integrate(function(x2v) dgumbel_biv(x1_val, x2v, k1 = 1, k2 = 1, theta = 0),
                     0, 10, rel.tol = 1e-5)$value
  outer_fn <- Vectorize(function(x1) inner(0, x1))
  intg <- stats::integrate(outer_fn, 0, 10, rel.tol = 1e-5)$value
  expect_equal(intg, 1, tolerance = 1e-3)
})

test_that("rgumbel_biv returns correct dimensions", {
  set.seed(1)
  dat <- rgumbel_biv(50, 1, 1, 0.3)
  expect_equal(dim(dat), c(50L, 2L))
  expect_true(all(dat >= 0))
})

test_that("pgumbel_biv gives CDF in [0,1]", {
  vals <- pgumbel_biv(seq(0.1, 3, 0.5), seq(0.1, 3, 0.5))
  expect_true(all(vals >= 0 & vals <= 1))
})

test_that("pfgm_biv satisfies boundary conditions", {
  expect_equal(pfgm_biv(0, 0.5, theta = 0.5), 0)
  expect_equal(pfgm_biv(0.5, 0, theta = 0.5), 0)
})

test_that("rfgm_biv returns matrix with values in [0,1]", {
  set.seed(2)
  dat <- rfgm_biv(100, theta = 0.5)
  expect_equal(ncol(dat), 2L)
  expect_true(all(dat >= 0 & dat <= 1))
})

test_that("dfgm_biv integrates to 1", {
  # Outer function must be Vectorized so stats::integrate can pass a vector.
  inner_fn <- function(x1_val)
    stats::integrate(function(x2) dfgm_biv(x1_val, x2, theta = 0.3),
                     0, 1, rel.tol = 1e-6)$value
  outer_fn <- Vectorize(inner_fn)
  val <- stats::integrate(outer_fn, 0, 1, rel.tol = 1e-6)$value
  expect_equal(val, 1, tolerance = 1e-4)
})

test_that("sschur_biv is symmetric", {
  v1 <- sschur_biv(0.5, 1.0, lambda = 1)
  v2 <- sschur_biv(1.0, 0.5, lambda = 1)
  expect_equal(v1, v2)
})

test_that("rschur_biv has non-negative entries", {
  set.seed(3)
  dat <- rschur_biv(50, lambda = 2)
  expect_true(all(dat >= 0))
})

test_that("pbivpower returns 0 when any argument is 0", {
  expect_equal(pbivpower(0, 0.5), 0)
  expect_equal(pbivpower(0.5, 0), 0)
})

test_that("rbivpower returns positive values", {
  set.seed(4)
  dat <- rbivpower(50, p1 = 2, p2 = 2)
  expect_true(all(dat >= 0))
})

# ==============================================================================
# Plotting utilities
# ==============================================================================

#' Plot Bivariate Laplace Transform of Residual Lives
#'
#' @description
#' Plots the star Laplace transform of residual lives
#' \eqn{L^*_{X_{t_1|t_2}}(s_1)} as a function of \eqn{t_1} for fixed
#' \eqn{s_1}, \eqn{t_2}.  Optionally overlays two distributions for
#' visual comparison of the BLt-rl order.
#'
#' @param surv_fn Joint survival function.  If a second distribution is to be
#'   overlaid, pass it as \code{surv_fn2}.
#' @param surv_fn2 Optional second survival function for comparison.
#' @param s1 Laplace parameter (default 1).
#' @param t2 Fixed second truncation age (default 0.5).
#' @param t1_grid Grid of first truncation ages.
#' @param k1,k2,theta Parameters for the default Gumbel distribution; used
#'   only when \code{surv_fn = NULL}.
#' @param xlab,ylab,main Plot labels.
#' @param col1,col2 Line colours.
#' @param lwd Line width.
#' @param legend_labels Length-2 character vector for legend (ignored if
#'   \code{surv_fn2 = NULL}).
#'
#' @return Invisibly returns the data frame used for plotting.
#'
#' @examples
#' sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.3)
#' sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 1, theta = 0.3)
#' plot_blt_residual(sX, sY, s1 = 1, t2 = 0.5,
#'                   legend_labels = c("k1=1", "k1=2"))
#'
#' @export
plot_blt_residual <- function(surv_fn, surv_fn2 = NULL,
                               s1 = 1, t2 = 0.5,
                               t1_grid = seq(0.1, 3, by = 0.1),
                               k1 = 1, k2 = 1, theta = 0,
                               xlab = expression(t[1]),
                               ylab = expression(L^"*"[X[t[1]*"|"*t[2]]](s[1])),
                               main = "Bivariate LT of Residual Lives",
                               col1 = "steelblue", col2 = "firebrick",
                               lwd = 2,
                               legend_labels = c("Distribution 1",
                                                 "Distribution 2")) {
  if (is.null(surv_fn))
    surv_fn <- function(x1, x2) sgumbel_biv(x1, x2, k1, k2, theta)

  vals1 <- sapply(t1_grid, function(t1) {
    tryCatch(blt_residual(s1, s1, t1, t2, surv_fn = surv_fn, star = TRUE)[1],
             error = function(e) NA_real_)
  })

  df <- data.frame(t1 = t1_grid, L1_star = vals1)

  graphics::plot(t1_grid, vals1, type = "l", col = col1, lwd = lwd,
                 xlab = xlab, ylab = ylab, main = main)
  graphics::grid(col = "lightgray")

  if (!is.null(surv_fn2)) {
    vals2 <- sapply(t1_grid, function(t1) {
      tryCatch(blt_residual(s1, s1, t1, t2, surv_fn = surv_fn2, star = TRUE)[1],
               error = function(e) NA_real_)
    })
    graphics::lines(t1_grid, vals2, col = col2, lwd = lwd, lty = 2)
    df$L1_star2 <- vals2
    graphics::legend("topright", legend = legend_labels,
                     col = c(col1, col2), lty = c(1, 2), lwd = lwd, bty = "n")
  }
  invisible(df)
}

#' Plot Bivariate Laplace Transform of Reversed Residual Lives
#'
#' @description
#' Plots the reversed-life Laplace transform \eqn{L_{t_1|t_2}(s_1)} as a
#' function of \eqn{t_1} for fixed \eqn{s_1} and \eqn{t_2}.
#'
#' @param cdf_fn Joint CDF function.
#' @param cdf_fn2 Optional second CDF for comparison.
#' @param s1 Laplace parameter.
#' @param t2 Fixed second truncation time.
#' @param t1_grid Grid of first truncation times.
#' @param theta FGM parameter (used if \code{cdf_fn = NULL}).
#' @param xlab,ylab,main Plot labels.
#' @param col1,col2 Line colours.
#' @param lwd Line width.
#' @param legend_labels Legend labels.
#'
#' @return Invisibly returns the data frame used for plotting.
#'
#' @examples
#' cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.2)
#' cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.7)
#' plot_blt_reversed(cX, cY, s1 = 1, t2 = 0.5,
#'                   legend_labels = c("theta=0.2", "theta=0.7"))
#'
#' @export
plot_blt_reversed <- function(cdf_fn, cdf_fn2 = NULL,
                               s1 = 1, t2 = 0.5,
                               t1_grid = seq(0.1, 0.9, by = 0.05),
                               theta = 0,
                               xlab = expression(t[1]),
                               ylab = expression(L[t[1]*"|"*t[2]](s[1])),
                               main = "Bivariate LT of Reversed Residual Lives",
                               col1 = "darkgreen", col2 = "darkorange",
                               lwd = 2,
                               legend_labels = c("Distribution 1",
                                                 "Distribution 2")) {
  if (is.null(cdf_fn))
    cdf_fn <- function(x1, x2) pfgm_biv(x1, x2, theta)

  vals1 <- sapply(t1_grid, function(t1) {
    tryCatch(blt_reversed(s1, s1, t1, t2, cdf_fn = cdf_fn)[1],
             error = function(e) NA_real_)
  })
  df <- data.frame(t1 = t1_grid, L1 = vals1)

  graphics::plot(t1_grid, vals1, type = "l", col = col1, lwd = lwd,
                 xlab = xlab, ylab = ylab, main = main)
  graphics::grid(col = "lightgray")

  if (!is.null(cdf_fn2)) {
    vals2 <- sapply(t1_grid, function(t1) {
      tryCatch(blt_reversed(s1, s1, t1, t2, cdf_fn = cdf_fn2)[1],
               error = function(e) NA_real_)
    })
    graphics::lines(t1_grid, vals2, col = col2, lwd = lwd, lty = 2)
    df$L12 <- vals2
    graphics::legend("topright", legend = legend_labels,
                     col = c(col1, col2), lty = c(1, 2), lwd = lwd, bty = "n")
  }
  invisible(df)
}

#' Plot Residual Entropy Generating Function
#'
#' @description
#' Plots the REGF \eqn{\mathcal{I}_\alpha(X;t)} as a function of \eqn{t} for
#' given \eqn{\alpha}, or as a function of \eqn{\alpha} for given \eqn{t}.
#'
#' @param dens_fn Density function.
#' @param surv_fn Survival function; computed if \code{NULL}.
#' @param alpha Positive parameter (used when \code{vary = "t"}).
#' @param t Truncation time (used when \code{vary = "alpha"}).
#' @param grid Numeric vector of the varying quantity values.
#' @param vary One of \code{"t"} (plot as function of \eqn{t}) or
#'   \code{"alpha"} (plot as function of \eqn{\alpha}).
#' @param upper Integration upper bound.
#' @param col Line colour.
#' @param lwd Line width.
#' @param main Plot title.
#'
#' @return Invisibly returns the data frame used for plotting.
#'
#' @examples
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#' plot_regf(f, Fb, alpha = 2, vary = "t")
#' plot_regf(f, Fb, t = 0.5, vary = "alpha")
#'
#' @export
plot_regf <- function(dens_fn, surv_fn = NULL,
                       alpha = 2, t = 0,
                       grid = NULL,
                       vary = c("t", "alpha"),
                       upper = 100,
                       col = "purple", lwd = 2,
                       main = NULL) {
  vary <- match.arg(vary)
  if (is.null(grid)) {
    grid <- if (vary == "t") seq(0, 3, by = 0.2) else seq(0.1, 3, by = 0.1)
  }

  vals <- if (vary == "t") {
    sapply(grid, function(ti)
      residual_info_gen(dens_fn, surv_fn, alpha = alpha, t = ti, upper = upper))
  } else {
    sapply(grid, function(a)
      residual_info_gen(dens_fn, surv_fn, alpha = a, t = t, upper = upper))
  }

  xlab_str <- if (vary == "t") "t" else expression(alpha)
  ylab_str <- expression(I[alpha](X * ";" * t))
  title_str <- if (is.null(main)) {
    if (vary == "t") paste0("REGF vs t  (alpha = ", alpha, ")")
    else paste0("REGF vs alpha  (t = ", t, ")")
  } else main

  graphics::plot(grid, vals, type = "l", col = col, lwd = lwd,
                 xlab = xlab_str, ylab = ylab_str, main = title_str)
  graphics::grid(col = "lightgray")

  invisible(data.frame(x = grid, REGF = vals))
}

# ==============================================================================
# Internal Utility Functions
# ==============================================================================

# Parameter checking helpers ---------------------------------------------------

.check_positive <- function(x, name) {
  if (!is.numeric(x) || length(x) != 1L || x <= 0)
    stop(sprintf("`%s` must be a single positive number.", name))
}

.check_nonneg <- function(x, name) {
  if (!is.numeric(x) || length(x) != 1L || x < 0)
    stop(sprintf("`%s` must be a single non-negative number.", name))
}

.check_range <- function(x, lo, hi, name) {
  if (!is.numeric(x) || length(x) != 1L || x < lo || x > hi)
    stop(sprintf("`%s` must be a single number in [%g, %g].", name, lo, hi))
}

# Numerical integration wrapper ------------------------------------------------

#' One-dimensional numerical integration using stats::integrate
#' @noRd
.integrate1d <- function(f, lower, upper, ..., tol = 1e-8, max_eval = 1e5) {
  if (lower >= upper) return(0)
  res <- tryCatch(
    stats::integrate(f, lower = lower, upper = upper,
                     rel.tol = tol, abs.tol = tol,
                     subdivisions = as.integer(max_eval), ...),
    error = function(e) list(value = NA_real_, message = conditionMessage(e))
  )
  if (is.na(res$value))
    warning("Numerical integration may have failed: ", res$message)
  res$value
}

# Empirical CDF / survival -----------------------------------------------------

#' Empirical bivariate survival function
#' @noRd
.biv_surv_empirical <- function(data, t1, t2) {
  mean(data[, 1] > t1 & data[, 2] > t2)
}

#' Empirical bivariate CDF
#' @noRd
.biv_cdf_empirical <- function(data, t1, t2) {
  mean(data[, 1] <= t1 & data[, 2] <= t2)
}

# Kernel density estimator (univariate, for REGF) ------------------------------

#' Kernel density at a grid of points (Gaussian, bandwidth = bw.nrd0)
#' @noRd
.kde_eval <- function(x, eval_pts, bw = stats::bw.nrd0(x)) {
  n <- length(x)
  sapply(eval_pts, function(t) mean(stats::dnorm((t - x) / bw)) / bw)
}

# Safe logarithm ---------------------------------------------------------------
.safe_log <- function(x) log(pmax(x, .Machine$double.eps))

# ==============================================================================
# Stochastic Ordering Functions
# References: Jayalekshmi, Rajesh, Nair (2022); Jayalekshmi, Rajesh (IJRQSE)
# ==============================================================================

# ------------------------------------------------------------------------------
# BLt-rl order (residual lives)
# ------------------------------------------------------------------------------

#' Bivariate Laplace Transform Order of Residual Lives (BLt-rl)
#'
#' @description
#' Checks whether random vector \eqn{X} is smaller than \eqn{Y} in the
#' bivariate Laplace transform order of residual lives (BLt-rl).
#'
#' \eqn{X \le_{\rm BLt\text{-}rl} Y} if and only if for all
#' \eqn{(t_1,t_2)} in the support:
#' \deqn{\frac{\int_{t_1}^\infty e^{-s_1 x_1}\bar{F}_X(x_1,t_2)\,dx_1}{
#'   e^{-s_1 t_1}\bar{F}_X(t_1,t_2)}
#'   \ge
#'   \frac{\int_{t_1}^\infty e^{-s_1 x_1}\bar{F}_Y(x_1,t_2)\,dx_1}{
#'   e^{-s_1 t_1}\bar{F}_Y(t_1,t_2)}}
#' i.e.\ \eqn{L^*_{X_{t_1|t_2}}(s_1) \ge L^*_{Y_{t_1|t_2}}(s_1)} for all
#' \eqn{t_1,t_2,s_1}.  The function evaluates this inequality at a grid.
#'
#' @param surv_X,surv_Y Joint survival functions for \eqn{X} and \eqn{Y}
#'   respectively, each of the form \code{function(x1, x2)}.
#' @param s1,s2 Laplace parameters to evaluate at.
#' @param t1_grid,t2_grid Grids of truncation times (default 0.1 to 3).
#' @param tol Tolerance for declaring inequality (default \code{1e-6}).
#'
#' @return A list with elements:
#'   \describe{
#'     \item{\code{order_holds}}{Logical; \code{TRUE} if \eqn{X \le Y} at all
#'       grid points.}
#'     \item{\code{violations}}{Data frame of grid points where the ordering
#'       fails.}
#'     \item{\code{ratio_matrix}}{Matrix of \eqn{L^*_X / L^*_Y} values.}
#'   }
#'
#' @examples
#' # Compare two Gumbel distributions
#' sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.2)
#' sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0.2)
#' blt_order_residual(sX, sY, s1 = 1, s2 = 1,
#'                    t1_grid = c(0.1, 0.5, 1), t2_grid = c(0.1, 0.5))
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022), Definition 4.1 & Prop. 4.1.
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @seealso \code{\link{blt_residual}}, \code{\link{biv_whr_order}}
#' @export
blt_order_residual <- function(surv_X, surv_Y, s1 = 1, s2 = 1,
                                t1_grid = seq(0.1, 3, by = 0.5),
                                t2_grid = seq(0.1, 3, by = 0.5),
                                tol = 1e-6) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")

  violations <- data.frame(t1 = numeric(0), t2 = numeric(0),
                            L1_star_X = numeric(0), L1_star_Y = numeric(0))
  ratio_mat <- matrix(NA_real_, nrow = length(t1_grid), ncol = length(t2_grid),
                      dimnames = list(
                        paste0("t1=", t1_grid),
                        paste0("t2=", t2_grid)
                      ))

  for (i in seq_along(t1_grid)) {
    for (j in seq_along(t2_grid)) {
      t1 <- t1_grid[i]; t2 <- t2_grid[j]

      Lx <- tryCatch(
        blt_residual(s1, s2, t1, t2, surv_fn = surv_X, star = TRUE)[1],
        error = function(e) NA_real_
      )
      Ly <- tryCatch(
        blt_residual(s1, s2, t1, t2, surv_fn = surv_Y, star = TRUE)[1],
        error = function(e) NA_real_
      )

      if (!is.na(Lx) && !is.na(Ly) && Ly > .Machine$double.eps)
        ratio_mat[i, j] <- Lx / Ly
      # X <= Y in BLt-rl iff L*_X >= L*_Y (star is reciprocal to L)
      if (!is.na(Lx) && !is.na(Ly) && (Lx < Ly - tol)) {
        violations <- rbind(violations, data.frame(
          t1 = t1, t2 = t2, L1_star_X = Lx, L1_star_Y = Ly
        ))
      }
    }
  }
  list(
    order_holds  = nrow(violations) == 0L,
    violations   = violations,
    ratio_matrix = ratio_mat
  )
}

# ------------------------------------------------------------------------------
# BLt-Rrl order (reversed residual lives)
# ------------------------------------------------------------------------------

#' Bivariate Laplace Transform Order of Reversed Residual Lives (BLt-Rrl)
#'
#' @description
#' Checks whether \eqn{X \le_{\rm BLt\text{-}Rrl} Y}: for all
#' \eqn{(t_1,t_2)}, \eqn{L^{X}_{t_1|t_2}(s_1) \ge L^{Y}_{t_1|t_2}(s_2)}.
#'
#' @param cdf_X,cdf_Y Joint CDF functions for \eqn{X} and \eqn{Y}.
#' @param s1,s2 Laplace parameters.
#' @param t1_grid,t2_grid Grids of truncation times.
#' @param tol Tolerance.
#'
#' @return Same structure as \code{\link{blt_order_residual}}.
#'
#' @examples
#' cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.3)
#' cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
#' blt_order_reversed(cX, cY, s1 = 1, s2 = 1,
#'                    t1_grid = c(0.2, 0.5), t2_grid = c(0.2, 0.5))
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Definition 2.
#'
#' @seealso \code{\link{blt_reversed}}, \code{\link{biv_wrhr_order}}
#' @export
blt_order_reversed <- function(cdf_X, cdf_Y, s1 = 1, s2 = 1,
                                t1_grid = seq(0.2, 0.8, by = 0.2),
                                t2_grid = seq(0.2, 0.8, by = 0.2),
                                tol = 1e-6) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")

  violations <- data.frame(t1 = numeric(0), t2 = numeric(0),
                            L1_X = numeric(0), L1_Y = numeric(0))

  for (t1 in t1_grid) {
    for (t2 in t2_grid) {
      Lx <- tryCatch(
        blt_reversed(s1, s2, t1, t2, cdf_fn = cdf_X)[1],
        error = function(e) NA_real_
      )
      Ly <- tryCatch(
        blt_reversed(s1, s2, t1, t2, cdf_fn = cdf_Y)[1],
        error = function(e) NA_real_
      )
      # X <= BLt-Rrl Y iff L_X >= L_Y
      if (!is.na(Lx) && !is.na(Ly) && (Lx < Ly - tol)) {
        violations <- rbind(violations,
                            data.frame(t1 = t1, t2 = t2, L1_X = Lx, L1_Y = Ly))
      }
    }
  }
  list(
    order_holds = nrow(violations) == 0L,
    violations  = violations
  )
}

# ------------------------------------------------------------------------------
# Weak bivariate hazard rate order (whr)
# ------------------------------------------------------------------------------

#' Weak Bivariate Hazard Rate Order
#'
#' @description
#' Checks whether \eqn{X \le_{\rm whr} Y}: the ratio
#' \eqn{\bar{F}_Y(x_1,x_2) / \bar{F}_X(x_1,x_2)} is increasing in
#' \eqn{(x_1,x_2)}, i.e.\ \eqn{h_{i,X}(t_1,t_2) \ge h_{i,Y}(t_1,t_2),\; i=1,2}.
#'
#' @param surv_X,surv_Y Joint survival functions.
#' @param t1_grid,t2_grid Evaluation grids.
#' @param tol Tolerance.
#'
#' @return A list: \code{order_holds} (logical), \code{violations} (data frame).
#'
#' @examples
#' sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0)
#' sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0)
#' biv_whr_order(sX, sY)
#'
#' @references
#' Shaked M., Shanthikumar J.G. (2007). *Stochastic Orders*. Springer.
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @export
biv_whr_order <- function(surv_X, surv_Y,
                           t1_grid = seq(0.1, 3, by = 0.5),
                           t2_grid = seq(0.1, 3, by = 0.5),
                           tol = 1e-6) {
  violations <- data.frame(t1 = numeric(0), t2 = numeric(0),
                            h1_X = numeric(0), h1_Y = numeric(0))

  for (t1 in t1_grid) {
    for (t2 in t2_grid) {
      hX <- biv_hazard_gradient(t1, t2, surv_fn = surv_X)
      hY <- biv_hazard_gradient(t1, t2, surv_fn = surv_Y)
      # X <=_whr Y iff h_i,X >= h_i,Y
      if (hX[1] < hY[1] - tol || hX[2] < hY[2] - tol)
        violations <- rbind(violations,
                            data.frame(t1 = t1, t2 = t2,
                                       h1_X = hX[1], h1_Y = hY[1]))
    }
  }
  list(order_holds = nrow(violations) == 0L, violations = violations)
}

# ------------------------------------------------------------------------------
# Weak bivariate MRL order (wmrl)
# ------------------------------------------------------------------------------

#' Weak Bivariate Mean Residual Life Order
#'
#' @description
#' Checks whether \eqn{X \le_{\rm wmrl} Y}:
#' \eqn{m_{i,X}(t_1,t_2) \le m_{i,Y}(t_1,t_2),\; i=1,2}.
#'
#' @param surv_X,surv_Y Joint survival functions.
#' @param t1_grid,t2_grid Evaluation grids.
#' @param tol Tolerance.
#'
#' @return List with \code{order_holds} and \code{violations}.
#'
#' @examples
#' sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1.5, k2 = 1.5)
#' sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
#' biv_wmrl_order(sX, sY)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @export
biv_wmrl_order <- function(surv_X, surv_Y,
                            t1_grid = seq(0.1, 2, by = 0.5),
                            t2_grid = seq(0.1, 2, by = 0.5),
                            tol = 1e-6) {
  violations <- data.frame(t1 = numeric(0), t2 = numeric(0),
                            m1_X = numeric(0), m1_Y = numeric(0))
  for (t1 in t1_grid) {
    for (t2 in t2_grid) {
      mX <- tryCatch(biv_mean_residual(t1, t2, surv_fn = surv_X),
                     error = function(e) c(m1 = NA, m2 = NA))
      mY <- tryCatch(biv_mean_residual(t1, t2, surv_fn = surv_Y),
                     error = function(e) c(m1 = NA, m2 = NA))
      if (!any(is.na(c(mX, mY))) &&
          (mX[1] > mY[1] + tol || mX[2] > mY[2] + tol))
        violations <- rbind(violations,
                            data.frame(t1 = t1, t2 = t2,
                                       m1_X = mX[1], m1_Y = mY[1]))
    }
  }
  list(order_holds = nrow(violations) == 0L, violations = violations)
}

# ------------------------------------------------------------------------------
# Bivariate relative MRL order (brlmr)
# ------------------------------------------------------------------------------

#' Bivariate Relative Mean Residual Life Order
#'
#' @description
#' Checks whether \eqn{X \le_{\rm brlmr} Y}: the ratio
#' \eqn{m_{i,Y}(t_1,t_2) / m_{i,X}(t_1,t_2)} is increasing in \eqn{t_i}.
#'
#' @param surv_X,surv_Y Joint survival functions.
#' @param t2_fixed Fixed value of \eqn{t_2} for the univariate slices.
#' @param t1_grid Grid of \eqn{t_1} values.
#' @param tol Tolerance.
#'
#' @return List with \code{order_holds} and \code{ratio_values}.
#'
#' @examples
#' sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 1)
#' sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
#' biv_brlmr_order(sX, sY, t2_fixed = 0.5)
#'
#' @references
#' Kayid M., Izadkhah S., Alshami S. (2016). Residual probability function,
#' associated orderings, and related aging classes. *Statistics and Probability
#' Letters*, 116, 37--47.
#'
#' @export
biv_brlmr_order <- function(surv_X, surv_Y, t2_fixed = 0.5,
                              t1_grid = seq(0.2, 3, by = 0.3), tol = 1e-6) {
  .check_nonneg(t2_fixed, "t2_fixed")
  ratios <- numeric(length(t1_grid))
  for (i in seq_along(t1_grid)) {
    t1 <- t1_grid[i]
    mX <- tryCatch(biv_mean_residual(t1, t2_fixed, surv_fn = surv_X)[1],
                   error = function(e) NA_real_)
    mY <- tryCatch(biv_mean_residual(t1, t2_fixed, surv_fn = surv_Y)[1],
                   error = function(e) NA_real_)
    ratios[i] <- if (is.na(mX) || mX <= 0) NA_real_ else mY / mX
  }
  increasing <- all(diff(ratios[!is.na(ratios)]) >= -tol)
  list(order_holds = increasing, ratio_values = ratios, t1_grid = t1_grid)
}

# ------------------------------------------------------------------------------
# Weak bivariate reversed hazard rate order (wrhr)
# ------------------------------------------------------------------------------

#' Weak Bivariate Reversed Hazard Rate Order
#'
#' @description
#' Checks \eqn{X \le_{\rm wrhr} Y}: the ratio
#' \eqn{F_X(x_1,x_2)/F_Y(x_1,x_2)} is decreasing in \eqn{(x_1,x_2)}, i.e.\
#' \eqn{h_{i,X}(x_1,x_2) \le h_{i,Y}(x_1,x_2),\; i=1,2}.
#'
#' @param cdf_X,cdf_Y Joint CDFs.
#' @param x1_grid,x2_grid Evaluation grids (positive values).
#' @param tol Tolerance.
#'
#' @return List with \code{order_holds} and \code{violations}.
#'
#' @examples
#' cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.2)
#' cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
#' biv_wrhr_order(cX, cY)
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
#'
#' @export
biv_wrhr_order <- function(cdf_X, cdf_Y,
                            x1_grid = seq(0.1, 0.9, by = 0.2),
                            x2_grid = seq(0.1, 0.9, by = 0.2),
                            tol = 1e-6) {
  violations <- data.frame(x1 = numeric(0), x2 = numeric(0),
                            rh1_X = numeric(0), rh1_Y = numeric(0))
  for (x1 in x1_grid) {
    for (x2 in x2_grid) {
      rhX <- biv_rhazard_gradient(x1, x2, cdf_fn = cdf_X)
      rhY <- biv_rhazard_gradient(x1, x2, cdf_fn = cdf_Y)
      if (rhX[1] > rhY[1] + tol || rhX[2] > rhY[2] + tol)
        violations <- rbind(violations,
                            data.frame(x1 = x1, x2 = x2,
                                       rh1_X = rhX[1], rh1_Y = rhY[1]))
    }
  }
  list(order_holds = nrow(violations) == 0L, violations = violations)
}

# ------------------------------------------------------------------------------
# Weak bivariate reversed MRL order (wrmrl)
# ------------------------------------------------------------------------------

#' Weak Bivariate Reversed Mean Residual Life Order
#'
#' @description
#' Checks \eqn{X \le_{\rm wrmrl} Y}:
#' \eqn{m_{i,X}(x_1,x_2) \ge m_{i,Y}(x_1,x_2),\; i=1,2}.
#'
#' @param cdf_X,cdf_Y Joint CDFs.
#' @param x1_grid,x2_grid Evaluation grids.
#' @param tol Tolerance.
#'
#' @return List with \code{order_holds} and \code{violations}.
#'
#' @examples
#' cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.1)
#' cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.8)
#' biv_wrmrl_order(cX, cY)
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
#'
#' @export
biv_wrmrl_order <- function(cdf_X, cdf_Y,
                              x1_grid = seq(0.2, 0.8, by = 0.2),
                              x2_grid = seq(0.2, 0.8, by = 0.2),
                              tol = 1e-6) {
  violations <- data.frame(x1 = numeric(0), x2 = numeric(0),
                            m1_X = numeric(0), m1_Y = numeric(0))
  for (x1 in x1_grid) {
    for (x2 in x2_grid) {
      mX <- tryCatch(biv_rmrl(x1, x2, cdf_fn = cdf_X),
                     error = function(e) c(m1 = NA, m2 = NA))
      mY <- tryCatch(biv_rmrl(x1, x2, cdf_fn = cdf_Y),
                     error = function(e) c(m1 = NA, m2 = NA))
      if (!any(is.na(c(mX, mY))) &&
          (mX[1] < mY[1] - tol || mX[2] < mY[2] - tol))
        violations <- rbind(violations,
                            data.frame(x1 = x1, x2 = x2,
                                       m1_X = mX[1], m1_Y = mY[1]))
    }
  }
  list(order_holds = nrow(violations) == 0L, violations = violations)
}

test_that("blt_order_residual: same distribution yields no violations", {
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.3)
  res <- blt_order_residual(sX, sX, s1 = 1, s2 = 1,
                             t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
  expect_true(res$order_holds)
})

test_that("blt_order_residual returns list with correct elements", {
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
  sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2)
  res <- blt_order_residual(sX, sY, t1_grid = 0.5, t2_grid = 0.5)
  expect_named(res, c("order_holds", "violations", "ratio_matrix"))
})

test_that("blt_order_reversed: same distribution yields no violations", {
  cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.4)
  res <- blt_order_reversed(cX, cX, s1 = 1, s2 = 1,
                             t1_grid = c(0.2, 0.5), t2_grid = c(0.2, 0.5))
  expect_true(res$order_holds)
})

test_that("biv_whr_order: X <=_whr Y when k_X > k_Y (Gumbel, theta=0)", {
  # Higher k => higher hazard => X <=_whr Y when h_X >= h_Y
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0)
  sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0)
  res <- biv_whr_order(sX, sY, t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
  expect_true(res$order_holds)
})

test_that("biv_whr_order returns correct structure", {
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
  sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
  res <- biv_whr_order(sX, sY, t1_grid = 0.5, t2_grid = 0.5)
  expect_named(res, c("order_holds", "violations"))
})

test_that("biv_wmrl_order: X <=_wmrl Y when X stoch smaller (Exp theta=0)", {
  # k_X > k_Y => E[X_i] < E[Y_i] => MRL_X <= MRL_Y
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0)
  sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0)
  res <- biv_wmrl_order(sX, sY, t1_grid = c(0.2, 0.5), t2_grid = c(0.2))
  expect_true(res$order_holds)
})

test_that("biv_brlmr_order: returns list", {
  sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
  sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 0.5, k2 = 0.5)
  res <- biv_brlmr_order(sX, sY, t2_fixed = 0.5, t1_grid = c(0.5, 1))
  expect_named(res, c("order_holds", "ratio_values", "t1_grid"))
})

test_that("biv_wrhr_order: same distribution holds", {
  cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.3)
  res <- biv_wrhr_order(cX, cX, x1_grid = c(0.2, 0.5), x2_grid = c(0.2, 0.5))
  expect_true(res$order_holds)
})

test_that("biv_wrmrl_order: same distribution holds", {
  cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
  res <- biv_wrmrl_order(cX, cX, x1_grid = c(0.3, 0.6), x2_grid = c(0.3))
  expect_true(res$order_holds)
})

# ==============================================================================
# Bivariate Laplace Transform of Reversed (Past) Residual Lives
# Reference: Jayalekshmi, Rajesh — IJRQSE
# ==============================================================================

# ------------------------------------------------------------------------------
# Core transform
# ------------------------------------------------------------------------------

#' Bivariate Laplace Transform of Reversed Residual Lives
#'
#' @description
#' Computes the bivariate Laplace transform of reversed (past) residual lives.
#' For a random pair \eqn{(X_1,X_2)} with joint distribution function
#' \eqn{F(x_1,x_2)}, the transform is
#' \deqn{L_{t_1|t_2}(s_1) = e^{-s_1 t_1} +
#'   \frac{s_1 \int_0^{t_1} e^{-s_1 x_1} F(x_1, t_2)\,dx_1}{F(t_1, t_2)}}
#' and the associated \eqn{G} function (useful for ordering) is
#' \deqn{G_1(t_1,t_2) =
#'   \frac{\int_0^{t_1} e^{-s_1 x_1} F(x_1, t_2)\,dx_1}{
#'     e^{-s_1 t_1} F(t_1,t_2)}.}
#'
#' @param s1,s2 Positive Laplace parameters.
#' @param t1,t2 Positive truncation times (\eqn{t_i > 0}).
#' @param cdf_fn A function \code{function(x1, x2)} returning the joint CDF
#'   \eqn{F(x_1,x_2)}.  Defaults to the FGM distribution.
#' @param theta FGM association parameter (used when \code{cdf_fn = NULL}).
#' @param g_form Logical; if \code{TRUE} returns the \eqn{G} form instead of
#'   \eqn{L}.
#'
#' @return A named numeric vector \code{(L1, L2)} or \code{(G1, G2)}.
#'
#' @examples
#' # FGM distribution (default)
#' blt_reversed(s1 = 1, s2 = 1, t1 = 0.6, t2 = 0.6)
#'
#' # G form
#' blt_reversed(s1 = 1, s2 = 1, t1 = 0.6, t2 = 0.6, g_form = TRUE)
#'
#' # User-supplied CDF
#' cdf <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
#' blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5, cdf_fn = cdf)
#'
#' @references
#' Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
#' reversed residual lives. *International Journal of Reliability, Quality and
#' Safety Engineering*.
#'
#' @seealso \code{\link{blt_reversed_fgm}}, \code{\link{blt_order_reversed}}
#' @export
blt_reversed <- function(s1, s2, t1, t2,
                          cdf_fn = NULL, theta = 0,
                          g_form = FALSE) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")
  .check_positive(t1, "t1"); .check_positive(t2, "t2")

  if (is.null(cdf_fn))
    cdf_fn <- function(x1, x2) pfgm_biv(x1, x2, theta)

  Ft1t2 <- cdf_fn(t1, t2)
  if (Ft1t2 <= 0) stop("cdf_fn(t1, t2) = 0; no mass at (t1, t2).")

  int1 <- .integrate1d(function(x1) exp(-s1 * x1) * cdf_fn(x1, t2), 0, t1)
  int2 <- .integrate1d(function(x2) exp(-s2 * x2) * cdf_fn(t1, x2), 0, t2)

  if (g_form) {
    G1 <- int1 / (exp(-s1 * t1) * Ft1t2)
    G2 <- int2 / (exp(-s2 * t2) * Ft1t2)
    return(c(G1 = G1, G2 = G2))
  }

  L1 <- exp(-s1 * t1) + s1 * int1 / Ft1t2
  L2 <- exp(-s2 * t2) + s2 * int2 / Ft1t2
  c(L1 = L1, L2 = L2)
}

# ------------------------------------------------------------------------------
# Closed-form: FGM distribution
# ------------------------------------------------------------------------------

#' Closed-Form BLT of Reversed Residual Lives for the FGM Distribution
#'
#' @description
#' Computes the bivariate Laplace transform of the FGM distribution in closed
#' form (Jayalekshmi and Rajesh, Example 1):
#' \deqn{L_X(s_1,s_2) = \frac{(1-e^{-s_1})(1-e^{-s_2})}{s_1 s_2}
#'   + \theta \Phi(s_1,s_2)}
#' where \eqn{\Phi} captures the dependence correction.
#'
#' @param s1,s2 Positive Laplace parameters.
#' @param theta FGM association parameter, \eqn{|\theta| \le 1}.
#'
#' @return Scalar numeric; the joint bivariate Laplace transform.
#'
#' @examples
#' blt_reversed_fgm(s1 = 1, s2 = 1, theta = 0.5)
#' blt_reversed_fgm(s1 = 2, s2 = 0.5, theta = -0.3)
#'
#' @references
#' Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
#' reversed residual lives. *International Journal of Reliability, Quality and
#' Safety Engineering*, Example 1.
#'
#' @export
blt_reversed_fgm <- function(s1, s2, theta = 0) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")
  .check_range(theta, -1, 1, "theta")

  phi0_s1 <- (1 - exp(-s1)) / s1
  phi0_s2 <- (1 - exp(-s2)) / s2

  # Phi correction: integral of theta*(1-2x1)*(1-2x2)*e^{-s1 x1 - s2 x2} dx1 dx2
  phi_s1 <- (1 - exp(-s1) - s1 * exp(-s1)) / s1^2   # int_0^1 (1-2u)e^{-su} du
  phi_s2 <- (1 - exp(-s2) - s2 * exp(-s2)) / s2^2

  phi0_s1 * phi0_s2 + theta * phi_s1 * phi_s2
}

# ------------------------------------------------------------------------------
# Closed-form: Bivariate power distribution
# ------------------------------------------------------------------------------

#' Bivariate Laplace Transform of Reversed Residual Lives — Power Distribution
#'
#' @description
#' Computes the \eqn{G_1} function for the bivariate power distribution:
#' \deqn{G_1(t_1,t_2) =
#'   \frac{\int_0^{t_1} e^{-s_1 x_1}
#'     x_1^{p_1+\theta\log t_2}\,dx_1}{e^{-s_1 t_1}\, t_1^{p_1+\theta\log t_2}}}
#' evaluated numerically.
#'
#' @param s1 Positive Laplace parameter.
#' @param t1,t2 Positive truncation times.
#' @param p1,p2 Positive shape parameters.
#' @param theta Association parameter, \eqn{0 \le \theta \le 1}.
#'
#' @return Named numeric vector \code{(G1, reversed_hazard_h1)}.
#'
#' @examples
#' blt_reversed_power(s1 = 1, t1 = 0.5, t2 = 0.5, p1 = 2, p2 = 2, theta = 0.3)
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Example 2.
#'
#' @export
blt_reversed_power <- function(s1, t1, t2, p1 = 1, p2 = 1, theta = 0) {
  .check_positive(s1, "s1")
  .check_positive(t1, "t1"); .check_positive(t2, "t2")
  .check_positive(p1, "p1"); .check_positive(p2, "p2")
  .check_range(theta, 0, 1, "theta")

  alpha <- p1 + theta * log(pmax(t2, .Machine$double.eps))
  Ft1t2 <- t1^alpha * t2^p2

  int1 <- .integrate1d(
    function(x1) exp(-s1 * x1) * x1^(p1 + theta * log(pmax(t2, .Machine$double.eps))),
    0, t1
  )
  G1 <- int1 / (exp(-s1 * t1) * Ft1t2)
  # Reversed hazard: h1(t1,t2) = s1 + 1/G1 - d/dt1 log G1
  # Approximate numerically
  eps <- 1e-6
  int1_p <- .integrate1d(
    function(x1) exp(-s1 * x1) *
      x1^(p1 + theta * log(pmax(t2, .Machine$double.eps))),
    0, t1 + eps
  )
  Ft1p <- (t1 + eps)^alpha * t2^p2
  G1p <- int1_p / (exp(-s1 * (t1 + eps)) * Ft1p)
  dlogG1dt1 <- (log(pmax(G1p, .Machine$double.eps)) -
                  log(pmax(G1, .Machine$double.eps))) / eps
  h1 <- s1 + 1 / pmax(G1, .Machine$double.eps) - dlogG1dt1

  c(G1 = G1, reversed_hazard_h1 = h1)
}

# ------------------------------------------------------------------------------
# Bivariate reversed hazard gradient
# ------------------------------------------------------------------------------

#' Bivariate Reversed Hazard Gradient
#'
#' @description
#' Computes the bivariate reversed hazard gradient
#' \deqn{h_i(x_1,x_2) = \frac{\partial}{\partial x_i}\log F(x_1,x_2),
#'   \quad i = 1, 2.}
#'
#' @param x1,x2 Positive evaluation points.
#' @param cdf_fn Joint CDF function; defaults to FGM with parameter
#'   \code{theta}.
#' @param theta FGM parameter (used when \code{cdf_fn = NULL}).
#' @param eps Numerical differentiation step.
#'
#' @return Named numeric vector \code{(rh1, rh2)}.
#'
#' @examples
#' biv_rhazard_gradient(x1 = 0.5, x2 = 0.5, theta = 0.3)
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
#'
#' @export
biv_rhazard_gradient <- function(x1, x2, cdf_fn = NULL, theta = 0,
                                  eps = 1e-7) {
  .check_positive(x1, "x1"); .check_positive(x2, "x2")
  if (is.null(cdf_fn))
    cdf_fn <- function(u1, u2) pfgm_biv(u1, u2, theta)

  logF <- function(u1, u2) .safe_log(cdf_fn(u1, u2))
  rh1 <- (logF(x1 + eps, x2) - logF(x1, x2)) / eps
  rh2 <- (logF(x1, x2 + eps) - logF(x1, x2)) / eps
  c(rh1 = rh1, rh2 = rh2)
}

# ------------------------------------------------------------------------------
# Bivariate reversed mean residual life
# ------------------------------------------------------------------------------

#' Bivariate Reversed Mean Residual Life Function
#'
#' @description
#' Computes the bivariate reversed mean residual life (RMRL):
#' \deqn{m_1(x_1,x_2) = \frac{\int_0^{x_1} F(t_1,x_2)\,dt_1}{F(x_1,x_2)},
#'   \quad m_2(x_1,x_2) = \frac{\int_0^{x_2} F(x_1,t_2)\,dt_2}{F(x_1,x_2)}.}
#'
#' @param x1,x2 Positive evaluation points.
#' @param cdf_fn Joint CDF; defaults to FGM.
#' @param theta FGM parameter.
#'
#' @return Named numeric vector \code{(m1, m2)}.
#'
#' @examples
#' biv_rmrl(x1 = 0.5, x2 = 0.5, theta = 0.3)
#' biv_rmrl(x1 = 0.7, x2 = 0.4, theta = -0.2)
#'
#' @references
#' Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
#'
#' @export
biv_rmrl <- function(x1, x2, cdf_fn = NULL, theta = 0) {
  .check_positive(x1, "x1"); .check_positive(x2, "x2")
  if (is.null(cdf_fn))
    cdf_fn <- function(u1, u2) pfgm_biv(u1, u2, theta)

  Fx1x2 <- cdf_fn(x1, x2)
  if (Fx1x2 <= 0) stop("cdf_fn(x1, x2) = 0.")

  m1 <- .integrate1d(function(t1) cdf_fn(t1, x2), 0, x1) / Fx1x2
  m2 <- .integrate1d(function(t2) cdf_fn(x1, t2), 0, x2) / Fx1x2
  c(m1 = m1, m2 = m2)
}

test_that("blt_reversed returns named vector", {
  res <- blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5)
  expect_named(res, c("L1", "L2"))
})

test_that("blt_reversed G form is positive", {
  res <- blt_reversed(s1 = 1, s2 = 1, t1 = 0.4, t2 = 0.4, g_form = TRUE)
  expect_named(res, c("G1", "G2"))
  expect_true(all(res > 0))
})

test_that("blt_reversed_fgm is positive for theta=0", {
  val <- blt_reversed_fgm(s1 = 1, s2 = 1, theta = 0)
  expect_gt(val, 0)
})

test_that("blt_reversed_fgm gives same result for theta=0 as product of marginals", {
  # When theta=0, joint LT = product of marginal Uniform[0,1] LTs
  # LT of Uniform[0,1] at s: (1 - e^{-s})/s
  s <- 1.5
  lt_marg <- (1 - exp(-s)) / s
  val <- blt_reversed_fgm(s1 = s, s2 = s, theta = 0)
  expect_equal(val, lt_marg^2, tolerance = 1e-8)
})

test_that("blt_reversed_power returns named vector", {
  res <- blt_reversed_power(s1 = 1, t1 = 0.5, t2 = 0.5, p1 = 2, p2 = 2, theta = 0.2)
  expect_named(res, c("G1", "reversed_hazard_h1"))
})

test_that("biv_rhazard_gradient is positive for FGM", {
  rh <- biv_rhazard_gradient(x1 = 0.5, x2 = 0.5, theta = 0.5)
  expect_true(all(rh > 0))
})

test_that("biv_rmrl is positive", {
  m <- biv_rmrl(x1 = 0.5, x2 = 0.5, theta = 0.3)
  expect_named(m, c("m1", "m2"))
  expect_true(all(m > 0))
})

test_that("biv_rmrl values <= x1 and x2 (RMRL <= current age)", {
  m <- biv_rmrl(x1 = 0.7, x2 = 0.8, theta = 0)
  expect_lte(m["m1"], 0.7)
  expect_lte(m["m2"], 0.8)
})

test_that("blt_reversed increases with t1 for FGM distribution", {
  # Generally, the past-life Laplace transform should be increasing in t
  vals <- sapply(seq(0.1, 0.8, 0.1), function(t1)
    blt_reversed(s1 = 1, s2 = 1, t1 = t1, t2 = 0.5)[1])
  # Not strictly necessary in all cases but for uniform-like marginals it holds
  expect_true(is.numeric(vals))
})

# ==============================================================================
# Bivariate Laplace Transform of Residual Lives
# Reference: Jayalekshmi, Rajesh, Nair (2022) doi:10.1080/03610926.2022.2085874
# ==============================================================================

# ------------------------------------------------------------------------------
# Core transform
# ------------------------------------------------------------------------------

#' Bivariate Laplace Transform of Residual Lives
#'
#' @description
#' Computes the bivariate Laplace transform of residual lives
#' \eqn{L_{X_{t_1|t_2}}(s_1)} and \eqn{L_{X_{t_2|t_1}}(s_2)} defined as
#' \deqn{L_{X_{t_1|t_2}}(s_1)
#'   = \frac{\int_{t_1}^{\infty} e^{-s_1 x_1}
#'     f(x_1 | X_2 > t_2)\,dx_1}{e^{-s_1 t_1} \bar{F}(t_1 | X_2 > t_2)}}
#' and analogously for the second component.
#'
#' The *star* version (used for ordering) is
#' \deqn{L^*_{X_{t_1|t_2}}(s_1) = \frac{1 - L_{X_{t_1|t_2}}(s_1)}{s_1}
#'   = \frac{\int_{t_1}^{\infty} e^{-s_1 x_1}
#'     \bar{F}(x_1, t_2)}{e^{-s_1 t_1}\bar{F}(t_1,t_2)}\,dx_1.}
#'
#' @param s1,s2 Positive Laplace parameters.
#' @param t1,t2 Non-negative time thresholds (truncation ages).
#' @param surv_fn A function \code{function(x1, x2)} returning the joint
#'   survival probability \eqn{\bar{F}(x_1, x_2)}.  Defaults to the
#'   Gumbel bivariate exponential with \code{k1}, \code{k2}, \code{theta}.
#' @param k1,k2 Rate parameters (used only when \code{surv_fn = NULL}).
#' @param theta Association parameter (used only when \code{surv_fn = NULL}).
#' @param upper Integration upper bound (default \code{50}).
#' @param star Logical; if \code{TRUE} returns the star version \eqn{L^*}.
#'
#' @return A named numeric vector with elements \code{L1} and \code{L2}
#'   (or \code{L1_star} and \code{L2_star} when \code{star = TRUE}).
#'
#' @examples
#' # Gumbel bivariate exponential, default parameters
#' blt_residual(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5)
#'
#' # Star version used in ordering
#' blt_residual(s1 = 0.5, s2 = 0.5, t1 = 1, t2 = 1, star = TRUE)
#'
#' # User-supplied survival function (Schur-constant exponential)
#' surv <- function(x1, x2) exp(-(x1 + x2))
#' blt_residual(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, surv_fn = surv)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @seealso \code{\link{blt_residual_gumbel}}, \code{\link{blt_order_residual}},
#'   \code{\link{np_blt_residual}}
#' @export
blt_residual <- function(s1, s2, t1 = 0, t2 = 0,
                          surv_fn = NULL,
                          k1 = 1, k2 = 1, theta = 0,
                          upper = 50,
                          star = FALSE) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")
  .check_nonneg(t1, "t1");   .check_nonneg(t2, "t2")

  if (is.null(surv_fn))
    surv_fn <- function(x1, x2) sgumbel_biv(x1, x2, k1, k2, theta)

  Ft1t2 <- surv_fn(t1, t2)
  if (Ft1t2 <= 0) stop("surv_fn(t1, t2) = 0; support exceeded.")

  if (!star) {
    # L1 = int_{t1}^{infty} e^{-s1*x1} f(x1|X2>t2) dx1
    #    = numerator / (e^{-s1*t1} * S(t1|X2>t2))
    # Numerically: e^{-s1*x1} * f(x1|X2>t2) integrand
    # f(x1|X2>t2) = -d/dx1 S(x1,t2) / S_2(t2)    [not needed with ratio form]
    # Use: L1 = int_{t1}^{infty} e^{-s1*x1} * (-d/dx1 S(x1,t2)/S(t1,t2)) dx1
    eps <- 1e-8
    f1 <- function(x1) {
      dSdx1 <- (surv_fn(x1, t2) - surv_fn(x1 + eps, t2)) / eps
      exp(-s1 * x1) * dSdx1 / Ft1t2
    }
    num1 <- .integrate1d(f1, t1, upper)
    L1   <- num1 / exp(-s1 * t1)

    f2 <- function(x2) {
      dSdx2 <- (surv_fn(t1, x2) - surv_fn(t1, x2 + eps)) / eps
      exp(-s2 * x2) * dSdx2 / Ft1t2
    }
    num2 <- .integrate1d(f2, t2, upper)
    L2   <- num2 / exp(-s2 * t2)

    c(L1 = L1, L2 = L2)
  } else {
    # L*1 = int_{t1}^{infty} e^{-s1*x1} * S(x1,t2) dx1 / (e^{-s1*t1} * S(t1,t2))
    f1star <- function(x1) exp(-s1 * x1) * surv_fn(x1, t2)
    num1   <- .integrate1d(f1star, t1, upper)
    L1star <- num1 / (exp(-s1 * t1) * Ft1t2)

    f2star <- function(x2) exp(-s2 * x2) * surv_fn(t1, x2)
    num2   <- .integrate1d(f2star, t2, upper)
    L2star <- num2 / (exp(-s2 * t2) * Ft1t2)

    c(L1_star = L1star, L2_star = L2star)
  }
}

# ------------------------------------------------------------------------------
# Closed-form: Gumbel bivariate exponential
# ------------------------------------------------------------------------------

#' Closed-Form Bivariate Laplace Transform of Residual Lives (Gumbel)
#'
#' @description
#' Returns the *star* bivariate Laplace transform of residual lives for the
#' Gumbel bivariate exponential distribution in closed form:
#' \deqn{L^*_{X_{t_1|t_2}}(s_1) = \frac{1}{k_1 + s_1 + \theta t_2},\quad
#'   L^*_{X_{t_2|t_1}}(s_2) = \frac{1}{k_2 + s_2 + \theta t_1}.}
#'
#' @param s1,s2 Positive Laplace parameters.
#' @param t1,t2 Non-negative truncation ages.
#' @param k1,k2 Positive rate parameters.
#' @param theta Non-negative association parameter.
#'
#' @return A named numeric vector \code{(L1_star, L2_star)}.
#'
#' @examples
#' blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5)
#' blt_residual_gumbel(s1 = 2, s2 = 0.5, t1 = 0, t2 = 1, k1 = 2, k2 = 1, theta = 0.3)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022), Example 3.1.
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @seealso \code{\link{blt_residual}}
#' @export
blt_residual_gumbel <- function(s1, s2, t1 = 0, t2 = 0,
                                 k1 = 1, k2 = 1, theta = 0) {
  .check_positive(s1, "s1"); .check_positive(s2, "s2")
  .check_nonneg(t1, "t1");   .check_nonneg(t2, "t2")
  .check_positive(k1, "k1"); .check_positive(k2, "k2")
  .check_nonneg(theta, "theta")

  L1 <- 1 / (k1 + s1 + theta * t2)
  L2 <- 1 / (k2 + s2 + theta * t1)
  c(L1_star = L1, L2_star = L2)
}

# ------------------------------------------------------------------------------
# Bivariate hazard gradient
# ------------------------------------------------------------------------------

#' Bivariate Hazard Gradient
#'
#' @description
#' Computes the bivariate hazard gradient
#' \deqn{h_i(t_1,t_2) = -\frac{\partial}{\partial t_i}\log\bar{F}(t_1,t_2),
#'   \quad i = 1, 2.}
#'
#' @param t1,t2 Evaluation points (non-negative).
#' @param surv_fn A function \code{function(x1, x2)} returning the joint
#'   survival probability.  Defaults to Gumbel bivariate exponential.
#' @param k1,k2,theta Parameters for the default Gumbel distribution.
#' @param eps Step size for numerical differentiation (default \code{1e-7}).
#'
#' @return A named numeric vector \code{(h1, h2)}.
#'
#' @examples
#' biv_hazard_gradient(t1 = 1, t2 = 1)
#' biv_hazard_gradient(t1 = 0.5, t2 = 0.5, k1 = 2, k2 = 1.5, theta = 0.3)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @export
biv_hazard_gradient <- function(t1, t2,
                                 surv_fn = NULL,
                                 k1 = 1, k2 = 1, theta = 0,
                                 eps = 1e-7) {
  .check_nonneg(t1, "t1"); .check_nonneg(t2, "t2")
  if (is.null(surv_fn))
    surv_fn <- function(x1, x2) sgumbel_biv(x1, x2, k1, k2, theta)

  log_S <- function(x1, x2) .safe_log(surv_fn(x1, x2))
  h1 <- -(log_S(t1 + eps, t2) - log_S(t1, t2)) / eps
  h2 <- -(log_S(t1, t2 + eps) - log_S(t1, t2)) / eps
  c(h1 = h1, h2 = h2)
}

# ------------------------------------------------------------------------------
# Bivariate mean residual life
# ------------------------------------------------------------------------------

#' Bivariate Mean Residual Life Function
#'
#' @description
#' Computes the bivariate mean residual life (MRL) function
#' \deqn{m_1(t_1,t_2) = E(X_{t_1|t_2}) = \frac{\int_{t_1}^{\infty}
#'   \bar{F}(x_1,t_2)\,dx_1}{\bar{F}(t_1,t_2)}}
#' and analogously \eqn{m_2(t_1,t_2)}.
#'
#' @param t1,t2 Non-negative thresholds.
#' @param surv_fn Joint survival function; defaults to Gumbel bivariate
#'   exponential.
#' @param k1,k2,theta Gumbel parameters (used when \code{surv_fn = NULL}).
#' @param upper Upper integration bound (default \code{100}).
#'
#' @return A named numeric vector \code{(m1, m2)}.
#'
#' @examples
#' biv_mean_residual(t1 = 0.5, t2 = 0.5)
#' biv_mean_residual(t1 = 1, t2 = 0.5, k1 = 1, k2 = 2, theta = 0.2)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022).
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @export
biv_mean_residual <- function(t1, t2,
                               surv_fn = NULL,
                               k1 = 1, k2 = 1, theta = 0,
                               upper = 100) {
  .check_nonneg(t1, "t1"); .check_nonneg(t2, "t2")
  if (is.null(surv_fn))
    surv_fn <- function(x1, x2) sgumbel_biv(x1, x2, k1, k2, theta)

  Ft1t2 <- surv_fn(t1, t2)
  if (Ft1t2 <= 0) stop("surv_fn(t1, t2) = 0; support exceeded.")

  m1 <- .integrate1d(function(x1) surv_fn(x1, t2), t1, upper) / Ft1t2
  m2 <- .integrate1d(function(x2) surv_fn(t1, x2), t2, upper) / Ft1t2
  c(m1 = m1, m2 = m2)
}

# ------------------------------------------------------------------------------
# NBUHR / NWUHR aging class
# ------------------------------------------------------------------------------

#' Test NBUHR / NWUHR Aging Class
#'
#' @description
#' Checks whether a bivariate lifetime distribution belongs to the NBUHR
#' (New Better than Used in Hazard Rate) or NWUHR (New Worse than Used) aging
#' class.  A distribution is NBUHR if
#' \deqn{h_1(0,t_2) \ge h_1(t_1,t_2)\;\text{ for all }t_1 > 0}
#' and similarly for the second component.  The function evaluates this at a
#' grid of \eqn{t_1} values.
#'
#' @param t2 Fixed value of the second age coordinate.
#' @param t1_grid Numeric vector of \eqn{t_1} values to check (default 0.1 to
#'   5 in steps of 0.1).
#' @param surv_fn Joint survival function; defaults to Gumbel bivariate
#'   exponential.
#' @param k1,k2,theta Gumbel parameters.
#'
#' @return A list with components:
#'   \describe{
#'     \item{\code{class1}}{Character: \code{"NBUHR"}, \code{"NWUHR"}, or
#'       \code{"neither"} for the first component.}
#'     \item{\code{class2}}{Same for the second component.}
#'     \item{\code{h1_grid}}{Numeric vector of \eqn{h_1(t_1,t_2)} values.}
#'     \item{\code{h2_grid}}{Numeric vector of \eqn{h_2(t_1,t_2)} values.}
#'   }
#'
#' @examples
#' nbuhr_test(t2 = 1, k1 = 1, k2 = 1, theta = 0.3)
#' nbuhr_test(t2 = 0.5,
#'            surv_fn = function(x1, x2) exp(-(x1 + x2)))
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022), Definition 3.2.
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @export
nbuhr_test <- function(t2 = 1,
                        t1_grid = seq(0.1, 5, by = 0.1),
                        surv_fn = NULL,
                        k1 = 1, k2 = 1, theta = 0) {
  .check_nonneg(t2, "t2")
  if (is.null(surv_fn))
    surv_fn <- function(x1, x2) sgumbel_biv(x1, x2, k1, k2, theta)

  h0 <- biv_hazard_gradient(0, t2, surv_fn = surv_fn)
  h_grid <- t(sapply(t1_grid, function(t1)
    biv_hazard_gradient(t1, t2, surv_fn = surv_fn)))
  h1_vals <- h_grid[, 1]; h2_vals <- h_grid[, 2]

  classify <- function(h0_i, h_vals) {
    if (all(h0_i >= h_vals)) "NBUHR"
    else if (all(h0_i <= h_vals)) "NWUHR"
    else "neither"
  }

  list(
    class1  = classify(h0["h1"], h1_vals),
    class2  = classify(h0["h2"], h2_vals),
    h1_grid = h1_vals,
    h2_grid = h2_vals,
    t1_grid = t1_grid
  )
}

# ------------------------------------------------------------------------------
# Nonparametric estimator
# ------------------------------------------------------------------------------

#' Nonparametric Estimator for the Bivariate Laplace Transform of Residual Lives
#'
#' @description
#' Given a bivariate sample \eqn{(X_{1i}, X_{2i}),\; i=1,\ldots,n}, estimates
#' \deqn{\hat{L}^*_{1}(s_1; t_1, t_2)
#'   = \frac{\sum_{i:X_{1i}>t_1,\,X_{2i}>t_2}
#'     \int_{t_1}^{X_{1i}} e^{-s_1 u}\,du}{e^{-s_1 t_1} \cdot
#'     \#\{X_{1i}>t_1,X_{2i}>t_2\}}}
#' and analogously for the second component, using the empirical survival
#' function as described in Jayalekshmi et al. (2022), Section 6.
#'
#' @param data A two-column numeric matrix or data frame with columns for
#'   \eqn{X_1} and \eqn{X_2}.
#' @param s1,s2 Positive Laplace parameters.
#' @param t1,t2 Non-negative truncation ages.
#'
#' @return A named numeric vector \code{(L1_hat, L2_hat)}.
#'
#' @examples
#' set.seed(123)
#' dat <- rgumbel_biv(200, k1 = 1, k2 = 1, theta = 0.5)
#' np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3)
#'
#' # Compare with closed form
#' blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, theta = 0.5)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022), Section 6.
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @seealso \code{\link{blt_residual_gumbel}}, \code{\link{sim_blt_residual}}
#' @export
np_blt_residual <- function(data, s1, s2, t1 = 0, t2 = 0) {
  if (!is.matrix(data)) data <- as.matrix(data)
  if (ncol(data) < 2L) stop("`data` must have at least 2 columns.")
  .check_positive(s1, "s1"); .check_positive(s2, "s2")
  .check_nonneg(t1, "t1");   .check_nonneg(t2, "t2")

  x1 <- data[, 1]; x2 <- data[, 2]

  idx <- which(x1 > t1 & x2 > t2)
  m   <- length(idx)
  if (m == 0L) {
    warning("No observations exceed (t1, t2); returning NA.")
    return(c(L1_hat = NA_real_, L2_hat = NA_real_))
  }

  # L1*: integral from t1 to x1i of e^{-s1 u} du for each i in idx
  int_vals1 <- (exp(-s1 * t1) - exp(-s1 * x1[idx])) / s1
  L1_hat <- sum(int_vals1) / (exp(-s1 * t1) * m)

  int_vals2 <- (exp(-s2 * t2) - exp(-s2 * x2[idx])) / s2
  L2_hat <- sum(int_vals2) / (exp(-s2 * t2) * m)

  c(L1_hat = L1_hat, L2_hat = L2_hat)
}

# ------------------------------------------------------------------------------
# Simulation study for the estimator
# ------------------------------------------------------------------------------

#' Monte-Carlo Simulation Study for the BLT Residual Estimator
#'
#' @description
#' Evaluates the performance of \code{\link{np_blt_residual}} via repeated
#' simulation from the Gumbel bivariate exponential distribution and compares
#' estimates to the closed-form \code{\link{blt_residual_gumbel}} values.
#' Returns bias, variance, and mean squared error (MSE).
#'
#' @param n_obs Sample size per replicate.
#' @param n_sim Number of simulation replicates.
#' @param s1,s2 Laplace parameters.
#' @param t1,t2 Truncation ages.
#' @param k1,k2,theta Gumbel parameters.
#' @param seed Random seed for reproducibility.
#'
#' @return A data frame with columns \code{component}, \code{true_value},
#'   \code{mean_est}, \code{bias}, \code{variance}, \code{mse}.
#'
#' @examples
#' sim_blt_residual(n_obs = 100, n_sim = 50, s1 = 1, s2 = 1,
#'                  t1 = 0.3, t2 = 0.3, k1 = 1, k2 = 1, theta = 0.5)
#'
#' @references
#' Jayalekshmi S., Rajesh G., Nair N.U. (2022), Section 6.
#' \doi{10.1080/03610926.2022.2085874}
#'
#' @seealso \code{\link{np_blt_residual}}
#' @export
sim_blt_residual <- function(n_obs = 200, n_sim = 100,
                              s1 = 1, s2 = 1,
                              t1 = 0.3, t2 = 0.3,
                              k1 = 1, k2 = 1, theta = 0.5,
                              seed = 42L) {
  set.seed(seed)
  true_vals <- blt_residual_gumbel(s1, s2, t1, t2, k1, k2, theta)

  ests <- replicate(n_sim, {
    dat <- rgumbel_biv(n_obs, k1, k2, theta)
    np_blt_residual(dat, s1, s2, t1, t2)
  })

  summarise_comp <- function(true, est_vec, label) {
    bias <- mean(est_vec, na.rm = TRUE) - true
    vr   <- stats::var(est_vec, na.rm = TRUE)
    data.frame(
      component   = label,
      true_value  = true,
      mean_est    = mean(est_vec, na.rm = TRUE),
      bias        = bias,
      variance    = vr,
      mse         = bias^2 + vr,
      stringsAsFactors = FALSE
    )
  }

  rbind(
    summarise_comp(true_vals[1], ests[1, ], "L1_star"),
    summarise_comp(true_vals[2], ests[2, ], "L2_star")
  )
}

test_that("blt_residual_gumbel matches paper formula", {
  # Example 3.1 from Jayalekshmi et al. (2022)
  # L*_1 = 1/(k1 + s1 + theta*t2)
  res <- blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
                              k1 = 1, k2 = 1, theta = 0.5)
  expect_equal(res["L1_star"], c(L1_star = 1 / (1 + 1 + 0.5 * 0.5)))
  expect_equal(res["L2_star"], c(L2_star = 1 / (1 + 1 + 0.5 * 0.5)))
})

test_that("blt_residual_gumbel is positive", {
  res <- blt_residual_gumbel(s1 = 2, s2 = 0.5, t1 = 1, t2 = 0.3,
                              k1 = 2, k2 = 1, theta = 0.3)
  expect_true(all(res > 0))
})

test_that("blt_residual numeric matches closed-form (independence case)", {
  # theta = 0 => independent exponentials; marginal LT of residual life
  # L*(s, t) = 1/(k + s) [independent of t for exponential]
  res_num <- blt_residual(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
                           k1 = 1, k2 = 1, theta = 0, star = TRUE)
  res_cl  <- blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
                                  k1 = 1, k2 = 1, theta = 0)
  expect_equal(as.numeric(res_num), as.numeric(res_cl), tolerance = 1e-3)
})

test_that("blt_residual returns named vector", {
  res <- blt_residual(1, 1, 0.5, 0.5, star = FALSE)
  expect_named(res, c("L1", "L2"))
  res2 <- blt_residual(1, 1, 0.5, 0.5, star = TRUE)
  expect_named(res2, c("L1_star", "L2_star"))
})

test_that("biv_hazard_gradient is positive", {
  hg <- biv_hazard_gradient(1, 1, k1 = 1, k2 = 1, theta = 0.3)
  expect_true(all(hg > 0))
})

test_that("biv_hazard_gradient has correct Gumbel form", {
  # h_i = k_i + theta * t_j for Gumbel distribution
  h <- biv_hazard_gradient(0.5, 1.0, k1 = 2, k2 = 1.5, theta = 0.3)
  expect_equal(as.numeric(h["h1"]), 2 + 0.3 * 1.0, tolerance = 1e-4)
  expect_equal(as.numeric(h["h2"]), 1.5 + 0.3 * 0.5, tolerance = 1e-4)
})

test_that("biv_mean_residual is positive", {
  mrl <- biv_mean_residual(0.5, 0.5, k1 = 1, k2 = 1, theta = 0)
  expect_true(all(mrl > 0))
})

test_that("biv_mean_residual matches analytic Exp for independence", {
  # When theta=0: m_i(t1,t2) = 1/k_i [memoryless property]
  mrl <- biv_mean_residual(1, 1, k1 = 2, k2 = 3, theta = 0)
  expect_equal(as.numeric(mrl["m1"]), 1/2, tolerance = 1e-3)
  expect_equal(as.numeric(mrl["m2"]), 1/3, tolerance = 1e-3)
})

test_that("nbuhr_test returns list with correct names", {
  res <- nbuhr_test(t2 = 0.5, k1 = 1, k2 = 1, theta = 0,
                    t1_grid = c(0.1, 0.5, 1))
  expect_named(res, c("class1", "class2", "h1_grid", "h2_grid", "t1_grid"))
})

test_that("nbuhr_test: Exp(k) marginal (theta=0) is NBUHR/NWUHR", {
  # Independent exponentials have constant hazard -> neither NBUHR nor NWUHR
  # (h is flat, so the ordering is an equality — classified as NBUHR by >=)
  res <- nbuhr_test(t2 = 0.5, k1 = 1, k2 = 1, theta = 0,
                    t1_grid = seq(0.1, 3, 0.5))
  expect_true(res$class1 %in% c("NBUHR", "NWUHR", "neither"))
})

test_that("np_blt_residual returns named numeric vector", {
  set.seed(10)
  dat <- rgumbel_biv(150, k1 = 1, k2 = 1, theta = 0.5)
  res <- np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.2, t2 = 0.2)
  expect_named(res, c("L1_hat", "L2_hat"))
  expect_true(all(res > 0))
})

test_that("np_blt_residual NA when no observations exceed threshold", {
  set.seed(11)
  dat <- rgumbel_biv(50, k1 = 1, k2 = 1)
  expect_warning(res <- np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 100, t2 = 100))
  expect_true(all(is.na(res)))
})

test_that("sim_blt_residual returns data frame with correct cols", {
  res <- sim_blt_residual(n_obs = 50, n_sim = 10, seed = 1)
  expect_s3_class(res, "data.frame")
  expect_true(all(c("component", "true_value", "bias", "mse") %in% names(res)))
})

# ==============================================================================
# Entropy and Information Generating Function
# Reference: Smitha S., Rajesh G., Jayalekshmi S. (2024) JISA 62(1):81-93
# ==============================================================================

# ------------------------------------------------------------------------------
# Shannon entropy
# ------------------------------------------------------------------------------

#' Shannon Differential Entropy
#'
#' @description
#' Computes the Shannon differential entropy
#' \deqn{H(f) = -\int_0^\infty f(x)\log f(x)\,dx}
#' for a non-negative continuous random variable with density \code{dens_fn}.
#'
#' @param dens_fn A function of one argument returning the density \eqn{f(x)}.
#' @param upper Upper integration limit (default 100).
#'
#' @return Scalar numeric.
#'
#' @examples
#' # Exponential(1): H = 1
#' shannon_entropy(function(x) dexp(x, rate = 1))
#'
#' # Exponential(2): H = 1 - log(2)
#' shannon_entropy(function(x) dexp(x, rate = 2))
#'
#' @references
#' Shannon C.E. (1948). A mathematical theory of communication. *Bell System
#' Technical Journal*, 27(3), 379--423.
#'
#' Smitha S., Rajesh G., Jayalekshmi S. (2024). *Journal of the Indian
#' Statistical Association*, 62(1), 81--93.
#'
#' @seealso \code{\link{residual_entropy}}, \code{\link{info_gen_function}}
#' @export
shannon_entropy <- function(dens_fn, upper = 100) {
  integrand <- function(x) {
    fx <- dens_fn(x)
    ifelse(fx > 0, -fx * log(fx), 0)
  }
  .integrate1d(integrand, 0, upper)
}

# ------------------------------------------------------------------------------
# Golomb information generating function
# ------------------------------------------------------------------------------

#' Golomb Information Generating Function
#'
#' @description
#' Computes the information generating function (IGF) introduced by Golomb
#' (1966):
#' \deqn{\mathcal{I}_\alpha(f) = \int_0^\infty f^\alpha(x)\,dx, \quad \alpha > 0.}
#'
#' When \eqn{\alpha \to 1},
#' \eqn{-d\mathcal{I}_\alpha / d\alpha|_{\alpha=1} = H(f)}.
#'
#' @param dens_fn A function of one argument returning the density.
#' @param alpha Positive parameter (default 1).
#' @param upper Upper integration limit.
#'
#' @return Scalar numeric.
#'
#' @examples
#' # Exponential(1) with alpha=1 gives 1
#' info_gen_function(function(x) dexp(x, rate = 1), alpha = 1)
#'
#' # alpha = 2
#' info_gen_function(function(x) dexp(x, rate = 1), alpha = 2)
#'
#' @references
#' Golomb S.W. (1966). The information generating function of a probability
#' distribution. *IEEE Transactions on Information Theory*, 12(1), 75--77.
#'
#' @seealso \code{\link{residual_info_gen}}
#' @export
info_gen_function <- function(dens_fn, alpha = 1, upper = 100) {
  .check_positive(alpha, "alpha")
  integrand <- function(x) {
    fx <- dens_fn(x)
    ifelse(fx > 0, fx^alpha, 0)
  }
  .integrate1d(integrand, 0, upper)
}

# ------------------------------------------------------------------------------
# Residual entropy (Ebrahimi & Pellerey 1995)
# ------------------------------------------------------------------------------

#' Residual Entropy Function
#'
#' @description
#' Computes the residual entropy (dynamic entropy) introduced by Ebrahimi and
#' Pellerey (1995):
#' \deqn{H(X; t) = -\int_t^\infty \frac{f(x)}{\bar{F}(t)}
#'   \log\frac{f(x)}{\bar{F}(t)}\,dx, \quad t \ge 0.}
#'
#' @param dens_fn Density function \eqn{f(x)}.
#' @param surv_fn Survival function \eqn{\bar{F}(x)}; if \code{NULL} it is
#'   computed from \code{dens_fn} by integration.
#' @param t Truncation time (default 0, which returns Shannon entropy).
#' @param upper Upper integration limit.
#'
#' @return Scalar numeric.
#'
#' @examples
#' # Exponential(1)
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#' residual_entropy(f, Fb, t = 0)   # = 1
#' residual_entropy(f, Fb, t = 1)   # = 1 + exp(-1) * log(exp(1)-1) or similar
#'
#' @references
#' Ebrahimi N., Pellerey F. (1995). New partial ordering of survival functions
#' based on the notion of uncertainty. *Journal of Applied Probability*, 32(1),
#' 202--211.
#'
#' @seealso \code{\link{residual_info_gen}}, \code{\link{shannon_entropy}}
#' @export
residual_entropy <- function(dens_fn, surv_fn = NULL, t = 0, upper = 100) {
  .check_nonneg(t, "t")
  if (is.null(surv_fn)) {
    surv_fn <- function(x) .integrate1d(dens_fn, x, upper)
  }
  Ft <- surv_fn(t)
  if (Ft <= 0) {
    warning("surv_fn(t) = 0; returning NA.")
    return(NA_real_)
  }
  integrand <- function(x) {
    fx <- dens_fn(x)
    ratio <- fx / Ft
    ifelse(ratio > 0, -ratio * log(ratio), 0)
  }
  .integrate1d(integrand, t, upper)
}

# ------------------------------------------------------------------------------
# Residual entropy generating function (REGF) — main new measure
# ------------------------------------------------------------------------------

#' Residual Entropy Generating Function (REGF)
#'
#' @description
#' Computes the residual entropy generating function proposed by Smitha,
#' Rajesh, and Jayalekshmi (2024):
#' \deqn{\mathcal{I}_\alpha(X; t) = \int_t^\infty
#'   \left(\frac{f(x)}{\bar{F}(t)}\right)^\alpha dx, \quad \alpha > 0,\; t \ge 0.}
#'
#' Note that:
#' \itemize{
#'   \item At \eqn{t = 0}, \eqn{\mathcal{I}_\alpha(X;0) = \mathcal{I}_\alpha(f)},
#'     Golomb's IGF.
#'   \item \eqn{-\partial \mathcal{I}_\alpha/\partial\alpha|_{\alpha=1} = H(X;t)},
#'     the residual entropy.
#' }
#'
#' @param dens_fn Density function \eqn{f(x)}.
#' @param surv_fn Survival function \eqn{\bar{F}(x)}; computed by integration
#'   if \code{NULL}.
#' @param alpha Positive parameter (default 1).
#' @param t Truncation time (default 0).
#' @param upper Upper integration limit.
#'
#' @return Scalar numeric.
#'
#' @examples
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#'
#' # At t=0, alpha=1: should equal 1
#' residual_info_gen(f, Fb, alpha = 1, t = 0)
#'
#' # Dynamic version at t = 0.5
#' residual_info_gen(f, Fb, alpha = 2, t = 0.5)
#'
#' # Derivative recovers residual entropy
#' (residual_info_gen(f, Fb, alpha = 1 + 1e-5, t = 0) -
#'   residual_info_gen(f, Fb, alpha = 1, t = 0)) / 1e-5   # approx -H(f)
#'
#' @references
#' Smitha S., Rajesh G., Jayalekshmi S. (2024). On residual entropy generating
#' function. *Journal of the Indian Statistical Association*, 62(1), 81--93.
#'
#' @seealso \code{\link{info_gen_function}}, \code{\link{residual_entropy}},
#'   \code{\link{np_residual_info_gen}}
#' @export
residual_info_gen <- function(dens_fn, surv_fn = NULL,
                               alpha = 1, t = 0, upper = 100) {
  .check_positive(alpha, "alpha")
  .check_nonneg(t, "t")
  if (is.null(surv_fn)) {
    surv_fn <- function(x) .integrate1d(dens_fn, x, upper)
  }
  Ft <- surv_fn(t)
  if (Ft <= 0) {
    warning("surv_fn(t) = 0; returning NA.")
    return(NA_real_)
  }
  integrand <- function(x) {
    fx <- dens_fn(x)
    (fx / Ft)^alpha
  }
  .integrate1d(integrand, t, upper)
}

# ------------------------------------------------------------------------------
# REGF profile across alpha
# ------------------------------------------------------------------------------

#' REGF Profile Over Alpha
#'
#' @description
#' Evaluates \code{\link{residual_info_gen}} over a grid of \eqn{\alpha} values
#' and optionally plots the result.  Useful for studying how the information
#' content varies with the Renyi-type parameter.
#'
#' @param dens_fn Density function.
#' @param surv_fn Survival function; computed internally if \code{NULL}.
#' @param alpha_grid Numeric vector of \eqn{\alpha} values (default 0.1 to 3).
#' @param t Truncation time.
#' @param upper Integration upper bound.
#' @param plot Logical; if \code{TRUE} plots the profile (default \code{FALSE}).
#'
#' @return A data frame with columns \code{alpha} and \code{REGF}.
#'
#' @examples
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#' regf_profile(f, Fb, t = 0.5, plot = FALSE)
#'
#' @export
regf_profile <- function(dens_fn, surv_fn = NULL,
                          alpha_grid = seq(0.1, 3, by = 0.2),
                          t = 0, upper = 100, plot = FALSE) {
  vals <- sapply(alpha_grid, function(a)
    residual_info_gen(dens_fn, surv_fn, alpha = a, t = t, upper = upper))
  result <- data.frame(alpha = alpha_grid, REGF = vals)
  if (plot) {
    graphics::plot(alpha_grid, vals, type = "l",
                   xlab = expression(alpha),
                   ylab = expression(I[alpha](X * ";" * t)),
                   main = paste0("REGF profile at t = ", t))
    graphics::grid()
  }
  invisible(result)
}

# ------------------------------------------------------------------------------
# Nonparametric estimator for REGF
# ------------------------------------------------------------------------------

#' Nonparametric Estimator for the Residual Entropy Generating Function
#'
#' @description
#' Given a univariate sample \eqn{X_1,\ldots,X_n}, estimates the REGF using a
#' kernel density estimator for \eqn{f} and the empirical survival function for
#' \eqn{\bar{F}(t)}:
#' \deqn{\hat{\mathcal{I}}_\alpha(t)
#'   = \sum_{i:X_i > t} \frac{\hat{f}(X_i)^{\alpha - 1}}{(n\,\hat{F}_n(t))^\alpha}
#'   \cdot \frac{1}{n}}
#' where \eqn{\hat{f}} is a Gaussian kernel density estimator.
#'
#' @param x Numeric vector; observed sample.
#' @param alpha Positive parameter (default 1).
#' @param t Truncation time (default 0).
#' @param bw Bandwidth for KDE; uses \code{\link[stats]{bw.nrd0}} if
#'   \code{NULL}.
#'
#' @return Scalar numeric estimate of REGF.
#'
#' @examples
#' set.seed(1)
#' x <- rexp(100, rate = 1)
#' np_residual_info_gen(x, alpha = 2, t = 0)
#'
#' # Compare with true value
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#' residual_info_gen(f, Fb, alpha = 2, t = 0)
#'
#' @references
#' Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 4.
#' *Journal of the Indian Statistical Association*, 62(1), 81--93.
#'
#' @seealso \code{\link{residual_info_gen}}, \code{\link{sim_regf}}
#' @export
np_residual_info_gen <- function(x, alpha = 1, t = 0, bw = NULL) {
  .check_positive(alpha, "alpha")
  .check_nonneg(t, "t")
  x <- x[!is.na(x)]
  n <- length(x)
  if (n < 5L) stop("Need at least 5 observations.")

  if (is.null(bw)) bw <- stats::bw.nrd0(x)
  Ft_hat <- mean(x > t)
  if (Ft_hat <= 0) {
    warning("No observations exceed t; returning NA.")
    return(NA_real_)
  }

  xi_gt <- x[x > t]
  if (length(xi_gt) == 0L) return(NA_real_)

  f_hat <- .kde_eval(x, xi_gt, bw = bw)
  # I_alpha(t) = int_{t}^{inf} (f(x)/Ft)^alpha dx
  # Approximated by: (1/n) * sum_{Xi > t} (f_hat(Xi)/Ft_hat)^alpha / f_hat(Xi)
  # = (1/n) * sum f_hat^(alpha-1) / Ft_hat^alpha
  sum((f_hat / Ft_hat)^alpha / pmax(f_hat, .Machine$double.eps) *
        (1 / n))
}

# ------------------------------------------------------------------------------
# Simulation study for REGF estimator
# ------------------------------------------------------------------------------

#' Monte-Carlo Simulation for the REGF Nonparametric Estimator
#'
#' @description
#' Evaluates the performance of \code{\link{np_residual_info_gen}} via repeated
#' simulation from the exponential distribution and compares to the true REGF
#' \eqn{\mathcal{I}_\alpha(\text{Exp}(\lambda); t) =
#'   \lambda^{\alpha-1} e^{(\alpha-1)\lambda t} / (\alpha-1)}.
#'
#' @param lambda Exponential rate.
#' @param alpha REGF parameter.
#' @param t Truncation time.
#' @param n_obs Sample size per replicate.
#' @param n_sim Number of replicates.
#' @param seed Random seed.
#'
#' @return A data frame with \code{true_value}, \code{mean_est}, \code{bias},
#'   \code{variance}, \code{mse}.
#'
#' @examples
#' sim_regf(lambda = 1, alpha = 2, t = 0, n_obs = 100, n_sim = 30)
#'
#' @references
#' Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 4.
#'
#' @seealso \code{\link{np_residual_info_gen}}
#' @export
sim_regf <- function(lambda = 1, alpha = 2, t = 0,
                      n_obs = 200, n_sim = 100, seed = 42L) {
  .check_positive(lambda, "lambda")
  .check_positive(alpha, "alpha")
  set.seed(seed)

  # True REGF for Exp(lambda): by the memoryless property, (X-t|X>t) ~ Exp(lambda)
  # so I_alpha(t) = integral_0^inf (lambda * e^{-lambda*u})^alpha du = lambda^(alpha-1)/alpha
  # This is constant in t for all alpha != 1.
  # At alpha = 1 the limit gives the Shannon residual entropy = log(1/lambda) + 1.
  true_val <- if (abs(alpha - 1) < 1e-10) {
    log(1 / lambda) + 1   # Shannon entropy of Exp(lambda)
  } else {
    lambda^(alpha - 1) / alpha
  }

  ests <- replicate(n_sim, {
    samp <- stats::rexp(n_obs, rate = lambda)
    np_residual_info_gen(samp, alpha = alpha, t = t)
  })

  bias <- mean(ests, na.rm = TRUE) - true_val
  vr   <- stats::var(ests, na.rm = TRUE)
  data.frame(
    true_value = true_val,
    mean_est   = mean(ests, na.rm = TRUE),
    bias       = bias,
    variance   = vr,
    mse        = bias^2 + vr
  )
}

# ------------------------------------------------------------------------------
# Distribution characterisation via REGF
# ------------------------------------------------------------------------------

#' Characterise a Distribution via the REGF
#'
#' @description
#' Checks whether the functional form of \eqn{\mathcal{I}_\alpha(X;t)} over a
#' grid of \eqn{t} values is consistent with an exponential distribution.  For
#' \eqn{X \sim \text{Exp}(\lambda)},
#' \eqn{\mathcal{I}_\alpha(t) / \mathcal{I}_\alpha(0) = e^{(\alpha-1)\lambda t}},
#' a pure exponential in \eqn{t}.
#'
#' @param dens_fn Density function.
#' @param surv_fn Survival function.
#' @param alpha REGF parameter.
#' @param t_grid Grid of truncation times.
#' @param upper Integration upper bound.
#'
#' @return A data frame with columns \code{t}, \code{REGF}, \code{log_REGF},
#'   and the fitted linear-in-\eqn{t} slope \code{slope} (log-linear
#'   characterisation).
#'
#' @examples
#' f  <- function(x) dexp(x, 1)
#' Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
#' regf_characterise(f, Fb, alpha = 2)
#'
#' @references
#' Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 3.
#'
#' @export
regf_characterise <- function(dens_fn, surv_fn = NULL, alpha = 2,
                               t_grid = seq(0, 2, by = 0.2),
                               upper = 100) {
  vals <- sapply(t_grid, function(t)
    residual_info_gen(dens_fn, surv_fn, alpha = alpha, t = t, upper = upper))
  log_vals <- .safe_log(vals / vals[1])
  # Fit log(I_alpha(t)) = beta * t via simple regression
  fit  <- if (sum(!is.na(log_vals)) > 1)
    unname(stats::lm(log_vals ~ t_grid)$coefficients[2])
  else NA_real_

  data.frame(t = t_grid, REGF = vals, log_REGF = log_vals, slope = fit,
             row.names = NULL)
}

test_that("shannon_entropy of Exp(1) equals 1", {
  h <- shannon_entropy(function(x) dexp(x, rate = 1))
  expect_equal(h, 1, tolerance = 1e-3)
})

test_that("shannon_entropy of Exp(lambda) equals 1 - log(lambda)", {
  for (lam in c(0.5, 1, 2)) {
    h <- shannon_entropy(function(x) dexp(x, rate = lam))
    expect_equal(h, 1 - log(lam), tolerance = 1e-3)
  }
})

test_that("info_gen_function of Exp(1) at alpha=1 equals 1", {
  val <- info_gen_function(function(x) dexp(x, rate = 1), alpha = 1)
  expect_equal(val, 1, tolerance = 1e-3)
})

test_that("info_gen_function of Exp(lambda) at alpha=2 = lambda/2", {
  # int_0^infty (lambda * e^{-lambda*x})^2 dx = lambda^2 / (2*lambda) = lambda/2
  for (lam in c(0.5, 1, 2)) {
    val <- info_gen_function(function(x) dexp(x, rate = lam), alpha = 2)
    expect_equal(val, lam / 2, tolerance = 1e-3)
  }
})

test_that("residual_entropy at t=0 equals Shannon entropy", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  re <- residual_entropy(f, Fb, t = 0)
  sh <- shannon_entropy(f)
  expect_equal(re, sh, tolerance = 1e-3)
})

test_that("residual_entropy is finite for Exp(1) at positive t", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  re <- residual_entropy(f, Fb, t = 1)
  expect_true(is.finite(re))
})

test_that("residual_entropy is non-negative", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  for (t in c(0, 0.5, 1, 2)) {
    expect_gte(residual_entropy(f, Fb, t = t), 0)
  }
})

test_that("residual_info_gen at t=0 equals info_gen_function", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  for (a in c(1, 2, 0.5)) {
    ri <- residual_info_gen(f, Fb, alpha = a, t = 0)
    ig <- info_gen_function(f, alpha = a)
    expect_equal(ri, ig, tolerance = 1e-3)
  }
})

test_that("residual_info_gen is constant in t for Exp(1), alpha=2", {
  # For Exp(lambda), the REGF = lambda^(alpha-1)/alpha for all t >= 0.
  # This follows from the memoryless property: the conditional distribution of
  # (X - t | X > t) is again Exp(lambda), so I_alpha(t) = I_alpha(0) = 1/alpha.
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  vals <- sapply(0:3, function(t) residual_info_gen(f, Fb, alpha = 2, t = t))
  # All values should equal 1/alpha = 0.5
  expect_equal(vals, rep(0.5, 4L), tolerance = 1e-3)
})

test_that("np_residual_info_gen returns positive scalar", {
  set.seed(42)
  x <- rexp(200, rate = 1)
  val <- np_residual_info_gen(x, alpha = 2, t = 0)
  expect_true(is.numeric(val) && length(val) == 1 && val > 0)
})

test_that("np_residual_info_gen warns when no data > t", {
  set.seed(5)
  x <- rexp(50, rate = 1)
  expect_warning(np_residual_info_gen(x, alpha = 1, t = 1000))
})

test_that("sim_regf returns data frame with correct columns", {
  res <- sim_regf(lambda = 1, alpha = 2, t = 0, n_obs = 50, n_sim = 10)
  expect_s3_class(res, "data.frame")
  expect_named(res, c("true_value", "mean_est", "bias", "variance", "mse"))
})

test_that("regf_profile returns data frame", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  res <- regf_profile(f, Fb, alpha_grid = c(0.5, 1, 2), t = 0)
  expect_s3_class(res, "data.frame")
  expect_named(res, c("alpha", "REGF"))
  expect_equal(nrow(res), 3)
})

test_that("regf_characterise returns data frame with correct columns", {
  f  <- function(x) dexp(x, 1)
  Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
  res <- regf_characterise(f, Fb, alpha = 2, t_grid = c(0, 0.5, 1))
  expect_s3_class(res, "data.frame")
  expect_named(res, c("t", "REGF", "log_REGF", "slope"))
})

---
title: "Introduction to BivLaplaceRL"
author: "Mahesh Divakaran"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to BivLaplaceRL}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
```

## Overview

**BivLaplaceRL** is an R package that implements statistical methods from two
research papers on bivariate Laplace transforms in reliability theory, together
with a set of univariate residual life tools:

1. Bivariate Laplace transform of residual lives (Jayalekshmi, Rajesh & Nair, 2022)
2. Bivariate Laplace transform order of reversed residual lives (Jayalekshmi & Rajesh)

```{r setup}
library(BivLaplaceRL)
```

## Parametric Distributions

The package provides four bivariate distributions commonly used in reliability
modelling.

### Gumbel Bivariate Exponential

```{r gumbel}
set.seed(42)
dat <- rgumbel_biv(200, k1 = 1, k2 = 1.5, theta = 0.3)
head(dat)
sgumbel_biv(1, 1, k1 = 1, k2 = 1.5, theta = 0.3)
```

### Farlie-Gumbel-Morgenstern (FGM)

```{r fgm}
set.seed(1)
dat_fgm <- rfgm_biv(100, theta = 0.5)
pfgm_biv(0.5, 0.6, theta = 0.5)
```

### Schur-Constant

```{r schur}
set.seed(2)
dat_sc <- rschur_biv(100, lambda = 1)
summary(rowSums(dat_sc))
```

## Bivariate Laplace Transform of Residual Lives

### Closed-Form (Gumbel Distribution)

$$L^*_{X_{t_1|t_2}}(s_1) = \frac{1}{k_1 + s_1 + \theta t_2}, \quad
  L^*_{X_{t_2|t_1}}(s_2) = \frac{1}{k_2 + s_2 + \theta t_1}.$$

```{r closed_form}
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
                    k1 = 1, k2 = 1, theta = 0.5)
```

### Numerical Computation

```{r numerical}
my_surv <- function(x1, x2) exp(-x1 - x2 - 0.3 * x1 * x2)
blt_residual(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
             surv_fn = my_surv, star = TRUE)
```

### Nonparametric Estimation

```{r np_estim}
set.seed(123)
dat <- rgumbel_biv(500, k1 = 1, k2 = 1, theta = 0.5)
np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3)

# True value
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, theta = 0.5)
```

### Simulation Study

```{r sim_study}
sim_blt_residual(n_obs = 200, n_sim = 50, s1 = 1, s2 = 1,
                 t1 = 0.3, t2 = 0.3, k1 = 1, k2 = 1, theta = 0.5)
```

## Bivariate Hazard Gradient and MRL

```{r hazard_mrl}
biv_hazard_gradient(t1 = 1, t2 = 1, k1 = 1, k2 = 1, theta = 0.3)
biv_mean_residual(t1 = 0.5, t2 = 0.5, k1 = 1, k2 = 1, theta = 0)
```

## NBUHR / NWUHR Aging Classification

```{r aging}
res <- nbuhr_test(t2 = 0.5, k1 = 1, k2 = 1, theta = 0.3,
                  t1_grid = seq(0.1, 3, 0.3))
cat("Component 1:", res$class1, "\n")
cat("Component 2:", res$class2, "\n")
```

## Bivariate Laplace Transform of Reversed Residual Lives

```{r reversed}
blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5, theta = 0.3)
biv_rhazard_gradient(x1 = 0.5, x2 = 0.5, theta = 0.3)
biv_rmrl(x1 = 0.5, x2 = 0.5, theta = 0.3)
```

## Bivariate Stochastic Orders

```{r blt_order}
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.2)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0.2)
res <- blt_order_residual(sX, sY, s1 = 1, s2 = 1,
                          t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
cat("X <=_BLt-rl Y:", res$order_holds, "\n")

res2 <- biv_whr_order(sX, sY, t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
cat("X <=_whr Y:", res2$order_holds, "\n")
```

## Univariate Residual Life Analysis

The package now provides a complete set of univariate residual life tools
that complement the bivariate framework.

### Laplace Transform of Residual Life

For a non-negative continuous random variable $X$ with density $f$ and
survival function $\bar{F}$:

$$L_X(s,t) = E[e^{-sX} \mid X > t]
  = \frac{1}{\bar{F}(t)}\int_t^\infty e^{-sx} f(x)\,dx.$$

```{r uni_lt}
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)

# At t=0: L_X(1, 0) = 1/(1+1) = 0.5
lt_residual(f, Fb, s = 1, t = 0)

# At t=0.5
lt_residual(f, Fb, s = 1, t = 0.5)
```

### Hazard Rate and MRL

```{r uni_hr_mrl}
# Exp(1): constant hazard rate = 1
hazard_rate(f, Fb, t = c(0.5, 1, 2))

# Exp(1): constant MRL = 1 (memoryless)
mean_residual(Fb, t = c(0, 0.5, 1, 2))

# Gamma(2,1): increasing hazard, decreasing MRL
fG  <- function(x) dgamma(x, shape = 2, rate = 1)
FbG <- function(x) pgamma(x, shape = 2, rate = 1, lower.tail = FALSE)
hazard_rate(fG, FbG, t = c(0.5, 1, 2))
mean_residual(FbG, t = c(0, 0.5, 1))
```

### Nonparametric Estimation

```{r uni_np}
set.seed(7)
x <- rexp(500, rate = 1)

# Nonparametric estimate at s=1, t=0: true value = 0.5
np_lt_residual(x, s = 1, t = 0)

# At t=0.5
np_lt_residual(x, s = 1, t = 0.5)
```

### Univariate Stochastic Orders

```{r uni_orders}
f1  <- function(x) dexp(x, 1)
Fb1 <- function(x) pexp(x, 1, lower.tail = FALSE)
f2  <- function(x) dexp(x, 2)
Fb2 <- function(x) pexp(x, 2, lower.tail = FALSE)

# LT-rl order: Exp(1) <=_Lt-rl Exp(2)?
lt_rl_order(f1, Fb1, f2, Fb2,
            s_grid = c(0.5, 1, 2), t_grid = c(0, 0.5, 1))$order_holds

# Hazard rate order: Exp(1) <=_hr Exp(2)?
hr_order(f1, Fb1, f2, Fb2, t_grid = c(0.5, 1, 2))$order_holds

# MRL order: Exp(2) <=_mrl Exp(1)?
mrl_order(Fb2, Fb1, t_grid = c(0, 0.5, 1, 2))$order_holds
```

## Entropy and Information Generating Functions

```{r entropy}
# Shannon entropy of Exp(1) = 1
shannon_entropy(function(x) dexp(x, 1))

# Golomb IGF of Exp(1) at alpha=2: 0.5
info_gen_function(function(x) dexp(x, 1), alpha = 2)
```

## References

Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform of
residual lives and their properties. *Communications in Statistics---Theory and
Methods*. <https://doi.org/10.1080/03610926.2022.2085874>

Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform order
and ordering of reversed residual lives. *International Journal of Reliability,
Quality and Safety Engineering*. <https://doi.org/10.1142/S0218539322500061>

Belzunce F., Ortega E., Ruiz J.M. (1999). The Laplace order and ordering of
residual lives. *Statistics & Probability Letters*, 42(2), 145--156.
<https://doi.org/10.1016/S0167-7152(98)00202-8>

---
title: "Introduction to BivLaplaceRL"
author: "Mahesh Divakaran"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to BivLaplaceRL}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
```

## Overview

**BivLaplaceRL** is an R package that implements statistical methods from three
research papers on bivariate Laplace transforms and entropy measures in
reliability theory:

1. Bivariate Laplace transform of residual lives (Jayalekshmi, Rajesh & Nair, 2022)
2. Bivariate Laplace transform order of reversed residual lives (Jayalekshmi & Rajesh)
3. Residual entropy generating function (Smitha, Rajesh & Jayalekshmi, 2024)

```{r setup}
library(BivLaplaceRL)
```

## Parametric Distributions

The package provides four bivariate distributions commonly used in reliability
modelling.

### Gumbel Bivariate Exponential

```{r gumbel}
# Simulate 200 observations
set.seed(42)
dat <- rgumbel_biv(200, k1 = 1, k2 = 1.5, theta = 0.3)
head(dat)

# Survival function at a point
sgumbel_biv(1, 1, k1 = 1, k2 = 1.5, theta = 0.3)
```

### Farlie-Gumbel-Morgenstern (FGM)

```{r fgm}
set.seed(1)
dat_fgm <- rfgm_biv(100, theta = 0.5)
head(dat_fgm)
pfgm_biv(0.5, 0.6, theta = 0.5)
```

### Schur-Constant

```{r schur}
set.seed(2)
dat_sc <- rschur_biv(100, lambda = 1)
# Marginals are standard exponential
summary(rowSums(dat_sc))  # T = X1 + X2 ~ Exp(lambda)
```

## Bivariate Laplace Transform of Residual Lives

### Closed-Form (Gumbel Distribution)

The key result from Jayalekshmi et al. (2022) is:

$$L^*_{X_{t_1|t_2}}(s_1) = \frac{1}{k_1 + s_1 + \theta t_2}, \quad
  L^*_{X_{t_2|t_1}}(s_2) = \frac{1}{k_2 + s_2 + \theta t_1}.$$

```{r closed_form}
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
                    k1 = 1, k2 = 1, theta = 0.5)
```

### Numerical Computation

For a user-supplied survival function:

```{r numerical}
my_surv <- function(x1, x2) exp(-x1 - x2 - 0.3 * x1 * x2)
blt_residual(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5,
             surv_fn = my_surv, star = TRUE)
```

### Nonparametric Estimation

```{r np_estim}
set.seed(123)
dat <- rgumbel_biv(500, k1 = 1, k2 = 1, theta = 0.5)
np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3)

# True value
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, theta = 0.5)
```

### Simulation Study

```{r sim_study}
sim_blt_residual(n_obs = 200, n_sim = 50, s1 = 1, s2 = 1,
                 t1 = 0.3, t2 = 0.3, k1 = 1, k2 = 1, theta = 0.5)
```

## Bivariate Hazard Gradient and MRL

```{r hazard_mrl}
# Hazard gradient
biv_hazard_gradient(t1 = 1, t2 = 1, k1 = 1, k2 = 1, theta = 0.3)

# Mean residual life
biv_mean_residual(t1 = 0.5, t2 = 0.5, k1 = 1, k2 = 1, theta = 0)
```

## NBUHR / NWUHR Aging Classification

```{r aging}
res <- nbuhr_test(t2 = 0.5, k1 = 1, k2 = 1, theta = 0.3,
                  t1_grid = seq(0.1, 3, 0.3))
cat("Component 1:", res$class1, "\n")
cat("Component 2:", res$class2, "\n")
```

## Bivariate Laplace Transform of Reversed Residual Lives

```{r reversed}
# FGM distribution
blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5, theta = 0.3)

# G form (used for characterisation)
blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5, theta = 0.3, g_form = TRUE)

# Reversed hazard gradient
biv_rhazard_gradient(x1 = 0.5, x2 = 0.5, theta = 0.3)

# Reversed MRL
biv_rmrl(x1 = 0.5, x2 = 0.5, theta = 0.3)
```

## Stochastic Orders

### BLt-rl Order

```{r blt_order}
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.2)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0.2)
res <- blt_order_residual(sX, sY, s1 = 1, s2 = 1,
                          t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
cat("X <=_BLt-rl Y:", res$order_holds, "\n")
```

### Weak Bivariate Hazard Rate Order

```{r whr_order}
res <- biv_whr_order(sX, sY, t1_grid = c(0.5, 1), t2_grid = c(0.5, 1))
cat("X <=_whr Y:", res$order_holds, "\n")
```

## Residual Entropy Generating Function

### Shannon Entropy and Golomb IGF

```{r entropy_basics}
# Shannon entropy of Exp(1) = 1
shannon_entropy(function(x) dexp(x, 1))

# Golomb IGF of Exp(1) at alpha=2: lambda/2 = 0.5
info_gen_function(function(x) dexp(x, 1), alpha = 2)
```

### Dynamic REGF

```{r regf}
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)

# REGF at various t values
t_vals <- seq(0, 2, 0.5)
sapply(t_vals, function(t) residual_info_gen(f, Fb, alpha = 2, t = t))
```

### Nonparametric Estimation

```{r np_regf}
set.seed(7)
x <- rexp(300, rate = 1)
np_residual_info_gen(x, alpha = 2, t = 0.5)

# True value
residual_info_gen(f, Fb, alpha = 2, t = 0.5)
```

### REGF Profile and Characterisation

```{r regf_profile, fig.cap="REGF profile across alpha"}
prof <- regf_profile(f, Fb, t = 0.5, plot = TRUE)
head(prof)
```

```{r regf_charac}
# Log-linearity in t characterises exponential distribution
ch <- regf_characterise(f, Fb, alpha = 2, t_grid = seq(0, 2, 0.4))
ch
```

### Simulation Study for REGF Estimator

```{r sim_regf}
sim_regf(lambda = 1, alpha = 2, t = 0, n_obs = 200, n_sim = 50)
```

## References

Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform of
residual lives and their properties. *Communications in Statistics—Theory and
Methods*. https://doi.org/10.1080/03610926.2022.2085874

Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*.

Smitha S., Rajesh G., Jayalekshmi S. (2024). On residual entropy generating
function. *Journal of the Indian Statistical Association*, 62(1), 81–93.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{np_residual_info_gen}
\alias{np_residual_info_gen}
\title{Nonparametric Estimator for the Residual Entropy Generating Function}
\usage{
np_residual_info_gen(x, alpha = 1, t = 0, bw = NULL)
}
\arguments{
\item{x}{Numeric vector; observed sample.}

\item{alpha}{Positive parameter (default 1).}

\item{t}{Truncation time (default 0).}

\item{bw}{Bandwidth for KDE; uses \code{\link[stats]{bw.nrd0}} if
\code{NULL}.}
}
\value{
Scalar numeric estimate of REGF.
}
\description{
Given a univariate sample \eqn{X_1,\ldots,X_n}, estimates the REGF using a
kernel density estimator for \eqn{f} and the empirical survival function for
\eqn{\bar{F}(t)}:
\deqn{\hat{\mathcal{I}}_\alpha(t)
  = \sum_{i:X_i > t} \frac{\hat{f}(X_i)^{\alpha - 1}}{(n\,\hat{F}_n(t))^\alpha}
  \cdot \frac{1}{n}}
where \eqn{\hat{f}} is a Gaussian kernel density estimator.
}
\examples{
set.seed(1)
x <- rexp(100, rate = 1)
np_residual_info_gen(x, alpha = 2, t = 0)

# Compare with true value
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
residual_info_gen(f, Fb, alpha = 2, t = 0)

}
\references{
Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 4.
*Journal of the Indian Statistical Association*, 62(1), 81--93.
}
\seealso{
\code{\link{residual_info_gen}}, \code{\link{sim_regf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{residual_info_gen}
\alias{residual_info_gen}
\title{Residual Entropy Generating Function (REGF)}
\usage{
residual_info_gen(dens_fn, surv_fn = NULL, alpha = 1, t = 0, upper = 100)
}
\arguments{
\item{dens_fn}{Density function \eqn{f(x)}.}

\item{surv_fn}{Survival function \eqn{\bar{F}(x)}; computed by integration
if \code{NULL}.}

\item{alpha}{Positive parameter (default 1).}

\item{t}{Truncation time (default 0).}

\item{upper}{Upper integration limit.}
}
\value{
Scalar numeric.
}
\description{
Computes the residual entropy generating function proposed by Smitha,
Rajesh, and Jayalekshmi (2024):
\deqn{\mathcal{I}_\alpha(X; t) = \int_t^\infty
  \left(\frac{f(x)}{\bar{F}(t)}\right)^\alpha dx, \quad \alpha > 0,\; t \ge 0.}

Note that:
\itemize{
  \item At \eqn{t = 0}, \eqn{\mathcal{I}_\alpha(X;0) = \mathcal{I}_\alpha(f)},
    Golomb's IGF.
  \item \eqn{-\partial \mathcal{I}_\alpha/\partial\alpha|_{\alpha=1} = H(X;t)},
    the residual entropy.
}
}
\examples{
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)

# At t=0, alpha=1: should equal 1
residual_info_gen(f, Fb, alpha = 1, t = 0)

# Dynamic version at t = 0.5
residual_info_gen(f, Fb, alpha = 2, t = 0.5)

# Derivative recovers residual entropy
(residual_info_gen(f, Fb, alpha = 1 + 1e-5, t = 0) -
  residual_info_gen(f, Fb, alpha = 1, t = 0)) / 1e-5   # approx -H(f)

}
\references{
Smitha S., Rajesh G., Jayalekshmi S. (2024). On residual entropy generating
function. *Journal of the Indian Statistical Association*, 62(1), 81--93.
}
\seealso{
\code{\link{info_gen_function}}, \code{\link{residual_entropy}},
  \code{\link{np_residual_info_gen}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{biv_hazard_gradient}
\alias{biv_hazard_gradient}
\title{Bivariate Hazard Gradient}
\usage{
biv_hazard_gradient(
  t1,
  t2,
  surv_fn = NULL,
  k1 = 1,
  k2 = 1,
  theta = 0,
  eps = 1e-07
)
}
\arguments{
\item{t1, t2}{Evaluation points (non-negative).}

\item{surv_fn}{A function \code{function(x1, x2)} returning the joint
survival probability.  Defaults to Gumbel bivariate exponential.}

\item{k1, k2, theta}{Parameters for the default Gumbel distribution.}

\item{eps}{Step size for numerical differentiation (default \code{1e-7}).}
}
\value{
A named numeric vector \code{(h1, h2)}.
}
\description{
Computes the bivariate hazard gradient
\deqn{h_i(t_1,t_2) = -\frac{\partial}{\partial t_i}\log\bar{F}(t_1,t_2),
  \quad i = 1, 2.}
}
\examples{
biv_hazard_gradient(t1 = 1, t2 = 1)
biv_hazard_gradient(t1 = 0.5, t2 = 0.5, k1 = 2, k2 = 1.5, theta = 0.3)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{shannon_entropy}
\alias{shannon_entropy}
\title{Shannon Differential Entropy}
\usage{
shannon_entropy(dens_fn, upper = 100)
}
\arguments{
\item{dens_fn}{A function of one argument returning the density \eqn{f(x)}.}

\item{upper}{Upper integration limit (default 100).}
}
\value{
Scalar numeric.
}
\description{
Computes the Shannon differential entropy
\deqn{H(f) = -\int_0^\infty f(x)\log f(x)\,dx}
for a non-negative continuous random variable with density \code{dens_fn}.
}
\examples{
# Exponential(1): H = 1
shannon_entropy(function(x) dexp(x, rate = 1))

# Exponential(2): H = 1 - log(2)
shannon_entropy(function(x) dexp(x, rate = 2))

}
\references{
Shannon C.E. (1948). A mathematical theory of communication. *Bell System
Technical Journal*, 27(3), 379--423.

Smitha S., Rajesh G., Jayalekshmi S. (2024). *Journal of the Indian
Statistical Association*, 62(1), 81--93.
}
\seealso{
\code{\link{residual_entropy}}, \code{\link{info_gen_function}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{biv_wmrl_order}
\alias{biv_wmrl_order}
\title{Weak Bivariate Mean Residual Life Order}
\usage{
biv_wmrl_order(
  surv_X,
  surv_Y,
  t1_grid = seq(0.1, 2, by = 0.5),
  t2_grid = seq(0.1, 2, by = 0.5),
  tol = 1e-06
)
}
\arguments{
\item{surv_X, surv_Y}{Joint survival functions.}

\item{t1_grid, t2_grid}{Evaluation grids.}

\item{tol}{Tolerance.}
}
\value{
List with \code{order_holds} and \code{violations}.
}
\description{
Checks whether \eqn{X \le_{\rm wmrl} Y}:
\eqn{m_{i,X}(t_1,t_2) \le m_{i,Y}(t_1,t_2),\; i=1,2}.
}
\examples{
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1.5, k2 = 1.5)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
biv_wmrl_order(sX, sY)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_reversed.R
\name{biv_rmrl}
\alias{biv_rmrl}
\title{Bivariate Reversed Mean Residual Life Function}
\usage{
biv_rmrl(x1, x2, cdf_fn = NULL, theta = 0)
}
\arguments{
\item{x1, x2}{Positive evaluation points.}

\item{cdf_fn}{Joint CDF; defaults to FGM.}

\item{theta}{FGM parameter.}
}
\value{
Named numeric vector \code{(m1, m2)}.
}
\description{
Computes the bivariate reversed mean residual life (RMRL):
\deqn{m_1(x_1,x_2) = \frac{\int_0^{x_1} F(t_1,x_2)\,dt_1}{F(x_1,x_2)},
  \quad m_2(x_1,x_2) = \frac{\int_0^{x_2} F(x_1,t_2)\,dt_2}{F(x_1,x_2)}.}
}
\examples{
biv_rmrl(x1 = 0.5, x2 = 0.5, theta = 0.3)
biv_rmrl(x1 = 0.7, x2 = 0.4, theta = -0.2)

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_reversed.R
\name{biv_rhazard_gradient}
\alias{biv_rhazard_gradient}
\title{Bivariate Reversed Hazard Gradient}
\usage{
biv_rhazard_gradient(x1, x2, cdf_fn = NULL, theta = 0, eps = 1e-07)
}
\arguments{
\item{x1, x2}{Positive evaluation points.}

\item{cdf_fn}{Joint CDF function; defaults to FGM with parameter
\code{theta}.}

\item{theta}{FGM parameter (used when \code{cdf_fn = NULL}).}

\item{eps}{Numerical differentiation step.}
}
\value{
Named numeric vector \code{(rh1, rh2)}.
}
\description{
Computes the bivariate reversed hazard gradient
\deqn{h_i(x_1,x_2) = \frac{\partial}{\partial x_i}\log F(x_1,x_2),
  \quad i = 1, 2.}
}
\examples{
biv_rhazard_gradient(x1 = 0.5, x2 = 0.5, theta = 0.3)

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{gumbel_biv}
\alias{gumbel_biv}
\alias{dgumbel_biv}
\alias{sgumbel_biv}
\alias{pgumbel_biv}
\alias{rgumbel_biv}
\title{Gumbel Bivariate Exponential Distribution}
\usage{
dgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0, log.p = FALSE)

sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0, log.p = FALSE)

pgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0)

rgumbel_biv(n, k1 = 1, k2 = 1, theta = 0)
}
\arguments{
\item{x1, x2}{Non-negative numeric values or vectors.}

\item{k1, k2}{Positive rate parameters.}

\item{theta}{Non-negative association parameter; must satisfy
\eqn{0 \le \theta \le k_1 k_2}.}

\item{log.p}{Logical; if \code{TRUE} probabilities are given as
\eqn{\log(p)}.}

\item{n}{Number of random observations.}
}
\value{
\describe{
  \item{\code{dgumbel_biv}}{Numeric vector of density values.}
  \item{\code{pgumbel_biv}}{Numeric vector of joint CDF values.}
  \item{\code{sgumbel_biv}}{Numeric vector of joint survival function values.}
  \item{\code{rgumbel_biv}}{A two-column matrix with columns \code{X1} and
    \code{X2} containing the simulated observations.}
}
}
\description{
Density, distribution (survival) function, and random generation for the
Gumbel bivariate exponential distribution with parameters \eqn{k_1},
\eqn{k_2}, and association parameter \eqn{\theta}.

The joint survival function is
\deqn{\bar{F}(x_1,x_2) = \exp(-k_1 x_1 - k_2 x_2 - \theta x_1 x_2),
  \quad x_1,x_2 > 0,\; k_1,k_2 > 0,\; 0 \le \theta \le k_1 k_2.}
}
\examples{
# Survival function
sgumbel_biv(1, 2, k1 = 1, k2 = 1, theta = 0.5)

# Density
dgumbel_biv(0.5, 0.5, k1 = 1, k2 = 1.5, theta = 0.3)

# Random sample
set.seed(42)
dat <- rgumbel_biv(100, k1 = 1, k2 = 1, theta = 0.5)
head(dat)

}
\references{
Gumbel E.J. (1960). Bivariate exponential distributions. *Journal of the
American Statistical Association*, 55(292), 698--707.

Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_blt_residual}
\alias{plot_blt_residual}
\title{Plot Bivariate Laplace Transform of Residual Lives}
\usage{
plot_blt_residual(
  surv_fn,
  surv_fn2 = NULL,
  s1 = 1,
  t2 = 0.5,
  t1_grid = seq(0.1, 3, by = 0.1),
  k1 = 1,
  k2 = 1,
  theta = 0,
  xlab = expression(t[1]),
  ylab = expression(L^"*"[X[t[1] * "|" * t[2]]](s[1])),
  main = "Bivariate LT of Residual Lives",
  col1 = "steelblue",
  col2 = "firebrick",
  lwd = 2,
  legend_labels = c("Distribution 1", "Distribution 2")
)
}
\arguments{
\item{surv_fn}{Joint survival function.  If a second distribution is to be
overlaid, pass it as \code{surv_fn2}.}

\item{surv_fn2}{Optional second survival function for comparison.}

\item{s1}{Laplace parameter (default 1).}

\item{t2}{Fixed second truncation age (default 0.5).}

\item{t1_grid}{Grid of first truncation ages.}

\item{k1, k2, theta}{Parameters for the default Gumbel distribution; used
only when \code{surv_fn = NULL}.}

\item{xlab, ylab, main}{Plot labels.}

\item{col1, col2}{Line colours.}

\item{lwd}{Line width.}

\item{legend_labels}{Length-2 character vector for legend (ignored if
\code{surv_fn2 = NULL}).}
}
\value{
Invisibly returns the data frame used for plotting.
}
\description{
Plots the star Laplace transform of residual lives
\eqn{L^*_{X_{t_1|t_2}}(s_1)} as a function of \eqn{t_1} for fixed
\eqn{s_1}, \eqn{t_2}.  Optionally overlays two distributions for
visual comparison of the BLt-rl order.
}
\examples{
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.3)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 1, theta = 0.3)
plot_blt_residual(sX, sY, s1 = 1, t2 = 0.5,
                  legend_labels = c("k1=1", "k1=2"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_reversed.R
\name{blt_reversed_power}
\alias{blt_reversed_power}
\title{Bivariate Laplace Transform of Reversed Residual Lives — Power Distribution}
\usage{
blt_reversed_power(s1, t1, t2, p1 = 1, p2 = 1, theta = 0)
}
\arguments{
\item{s1}{Positive Laplace parameter.}

\item{t1, t2}{Positive truncation times.}

\item{p1, p2}{Positive shape parameters.}

\item{theta}{Association parameter, \eqn{0 \le \theta \le 1}.}
}
\value{
Named numeric vector \code{(G1, reversed_hazard_h1)}.
}
\description{
Computes the \eqn{G_1} function for the bivariate power distribution:
\deqn{G_1(t_1,t_2) =
  \frac{\int_0^{t_1} e^{-s_1 x_1}
    x_1^{p_1+\theta\log t_2}\,dx_1}{e^{-s_1 t_1}\, t_1^{p_1+\theta\log t_2}}}
evaluated numerically.
}
\examples{
blt_reversed_power(s1 = 1, t1 = 0.5, t2 = 0.5, p1 = 2, p2 = 2, theta = 0.3)

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Example 2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_reversed.R
\name{blt_reversed_fgm}
\alias{blt_reversed_fgm}
\title{Closed-Form BLT of Reversed Residual Lives for the FGM Distribution}
\usage{
blt_reversed_fgm(s1, s2, theta = 0)
}
\arguments{
\item{s1, s2}{Positive Laplace parameters.}

\item{theta}{FGM association parameter, \eqn{|\theta| \le 1}.}
}
\value{
Scalar numeric; the joint bivariate Laplace transform.
}
\description{
Computes the bivariate Laplace transform of the FGM distribution in closed
form (Jayalekshmi and Rajesh, Example 1):
\deqn{L_X(s_1,s_2) = \frac{(1-e^{-s_1})(1-e^{-s_2})}{s_1 s_2}
  + \theta \Phi(s_1,s_2)}
where \eqn{\Phi} captures the dependence correction.
}
\examples{
blt_reversed_fgm(s1 = 1, s2 = 1, theta = 0.5)
blt_reversed_fgm(s1 = 2, s2 = 0.5, theta = -0.3)

}
\references{
Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*, Example 1.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BivLaplaceRL-package.R
\docType{package}
\name{BivLaplaceRL-package}
\alias{BivLaplaceRL-package}
\alias{BivLaplaceRL}
\title{BivLaplaceRL: Bivariate Laplace Transforms, Stochastic Orders, and Entropy
Measures in Reliability}
\description{
\pkg{BivLaplaceRL} provides a unified framework for three interconnected
research themes in reliability analysis and survival theory:

\strong{1. Bivariate Laplace Transform of Residual Lives}

Implements the vector Laplace transform of bivariate residual lives
\eqn{(L_{X_{t_1|t_2}}(s_1),\,L_{X_{t_2|t_1}}(s_2))}, the associated
stochastic ordering BLt-rl, and its relationships with the weak bivariate
hazard rate order, weak bivariate mean residual life order, and bivariate
relative mean residual life order.  Nonparametric estimation and NBUHR/NWUHR
aging class tests are included.
Reference: Jayalekshmi, Rajesh, and Nair (2022)
\doi{10.1080/03610926.2022.2085874}.

\strong{2. Bivariate Laplace Transform of Reversed Residual Lives}

Implements the bivariate Laplace transform of reversed (past) residual lives,
the BLt-Rrl stochastic order, and connections with weak bivariate reversed
hazard rate and reversed mean residual life orders.

\strong{3. Residual Entropy Generating Function}

Implements the dynamic information generating function (REGF), the residual
entropy of Ebrahimi and Pellerey (1995), Golomb's (1966) information
generating function, and nonparametric estimation of REGF with Monte-Carlo
simulation support.
}
\section{Parametric families}{

Gumbel bivariate exponential, Farlie-Gumbel-Morgenstern (FGM), bivariate
power, Schur-constant, and exponential distributions are built in.
}

\section{Plotting}{

\code{\link{plot_blt_residual}}, \code{\link{plot_blt_reversed}}, and
\code{\link{plot_regf}} provide ready-made visualisations.
}

\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022). Bivariate Laplace transform of
residual lives and their properties. *Communications in Statistics - Theory
and Methods*. \doi{10.1080/03610926.2022.2085874}

Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*.

Smitha S., Rajesh G., Jayalekshmi S. (2024). On residual entropy generating
function. *Journal of the Indian Statistical Association*, 62(1), 81--93.

Belzunce F., Ortega E., Ruiz J.M. (1999). The Laplace order and ordering of
residual lives. *Statistics & Probability Letters*, 42(2), 145--156.

Ebrahimi N., Pellerey F. (1995). New partial ordering of survival functions
based on the notion of uncertainty. *Journal of Applied Probability*, 32(1),
202--211.

Golomb S.W. (1966). The information generating function of a probability
distribution. *IEEE Transactions on Information Theory*, 12(1), 75--77.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://itsmdivakaran.github.io/BivLaplaceRL/}
  \item \url{https://github.com/itsmdivakaran/BivLaplaceRL}
  \item Report bugs at \url{https://github.com/itsmdivakaran/BivLaplaceRL/issues}
}

}
\author{
\strong{Maintainer}: Mahesh Divakaran \email{imaheshdivakaran@gmail.com} (\href{https://orcid.org/0000-0002-3488-0857}{ORCID}) (Amity School of Applied Sciences, Amity University Lucknow)

Authors:
\itemize{
  \item S. Jayalekshmi (Department of Statistics, Cochin University of Science and Technology)
  \item G. Rajesh \email{rajeshgstat@gmail.com} (Department of Statistics, Cochin University of Science and Technology)
  \item N. Unnikrishnan Nair (Department of Statistics, Cochin University of Science and Technology)
  \item Smitha S. (K. E. College, Mannanam)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{schur_biv}
\alias{schur_biv}
\alias{sschur_biv}
\alias{rschur_biv}
\title{Schur-Constant Bivariate Distribution}
\usage{
sschur_biv(x1, x2, lambda = 1)

rschur_biv(n, lambda = 1)
}
\arguments{
\item{x1, x2}{Non-negative values.}

\item{lambda}{Exponential rate parameter for the generating survival function.}

\item{n}{Number of random observations.}
}
\value{
Numeric vector (\code{sschur_biv}) or two-column matrix
  (\code{rschur_biv}).
}
\description{
Random generation and survival function for a Schur-constant bivariate
distribution with survival function
\deqn{\bar{F}(x_1,x_2) = S(x_1 + x_2),\quad x_1,x_2 > 0,}
where \eqn{S} is a given univariate survival function.  The default marginal
is exponential with rate \code{lambda}.
}
\examples{
sschur_biv(0.5, 1, lambda = 1)
set.seed(2); head(rschur_biv(40, lambda = 1))

}
\references{
Barlow R.E., Mendel M.B. (1992). De Finetti-type representations for life
distributions. *Journal of the American Statistical Association*, 87(420),
1116--1122.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{regf_profile}
\alias{regf_profile}
\title{REGF Profile Over Alpha}
\usage{
regf_profile(
  dens_fn,
  surv_fn = NULL,
  alpha_grid = seq(0.1, 3, by = 0.2),
  t = 0,
  upper = 100,
  plot = FALSE
)
}
\arguments{
\item{dens_fn}{Density function.}

\item{surv_fn}{Survival function; computed internally if \code{NULL}.}

\item{alpha_grid}{Numeric vector of \eqn{\alpha} values (default 0.1 to 3).}

\item{t}{Truncation time.}

\item{upper}{Integration upper bound.}

\item{plot}{Logical; if \code{TRUE} plots the profile (default \code{FALSE}).}
}
\value{
A data frame with columns \code{alpha} and \code{REGF}.
}
\description{
Evaluates \code{\link{residual_info_gen}} over a grid of \eqn{\alpha} values
and optionally plots the result.  Useful for studying how the information
content varies with the Renyi-type parameter.
}
\examples{
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
regf_profile(f, Fb, t = 0.5, plot = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{info_gen_function}
\alias{info_gen_function}
\title{Golomb Information Generating Function}
\usage{
info_gen_function(dens_fn, alpha = 1, upper = 100)
}
\arguments{
\item{dens_fn}{A function of one argument returning the density.}

\item{alpha}{Positive parameter (default 1).}

\item{upper}{Upper integration limit.}
}
\value{
Scalar numeric.
}
\description{
Computes the information generating function (IGF) introduced by Golomb
(1966):
\deqn{\mathcal{I}_\alpha(f) = \int_0^\infty f^\alpha(x)\,dx, \quad \alpha > 0.}

When \eqn{\alpha \to 1},
\eqn{-d\mathcal{I}_\alpha / d\alpha|_{\alpha=1} = H(f)}.
}
\examples{
# Exponential(1) with alpha=1 gives 1
info_gen_function(function(x) dexp(x, rate = 1), alpha = 1)

# alpha = 2
info_gen_function(function(x) dexp(x, rate = 1), alpha = 2)

}
\references{
Golomb S.W. (1966). The information generating function of a probability
distribution. *IEEE Transactions on Information Theory*, 12(1), 75--77.
}
\seealso{
\code{\link{residual_info_gen}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{biv_mean_residual}
\alias{biv_mean_residual}
\title{Bivariate Mean Residual Life Function}
\usage{
biv_mean_residual(
  t1,
  t2,
  surv_fn = NULL,
  k1 = 1,
  k2 = 1,
  theta = 0,
  upper = 100
)
}
\arguments{
\item{t1, t2}{Non-negative thresholds.}

\item{surv_fn}{Joint survival function; defaults to Gumbel bivariate
exponential.}

\item{k1, k2, theta}{Gumbel parameters (used when \code{surv_fn = NULL}).}

\item{upper}{Upper integration bound (default \code{100}).}
}
\value{
A named numeric vector \code{(m1, m2)}.
}
\description{
Computes the bivariate mean residual life (MRL) function
\deqn{m_1(t_1,t_2) = E(X_{t_1|t_2}) = \frac{\int_{t_1}^{\infty}
  \bar{F}(x_1,t_2)\,dx_1}{\bar{F}(t_1,t_2)}}
and analogously \eqn{m_2(t_1,t_2)}.
}
\examples{
biv_mean_residual(t1 = 0.5, t2 = 0.5)
biv_mean_residual(t1 = 1, t2 = 0.5, k1 = 1, k2 = 2, theta = 0.2)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{residual_entropy}
\alias{residual_entropy}
\title{Residual Entropy Function}
\usage{
residual_entropy(dens_fn, surv_fn = NULL, t = 0, upper = 100)
}
\arguments{
\item{dens_fn}{Density function \eqn{f(x)}.}

\item{surv_fn}{Survival function \eqn{\bar{F}(x)}; if \code{NULL} it is
computed from \code{dens_fn} by integration.}

\item{t}{Truncation time (default 0, which returns Shannon entropy).}

\item{upper}{Upper integration limit.}
}
\value{
Scalar numeric.
}
\description{
Computes the residual entropy (dynamic entropy) introduced by Ebrahimi and
Pellerey (1995):
\deqn{H(X; t) = -\int_t^\infty \frac{f(x)}{\bar{F}(t)}
  \log\frac{f(x)}{\bar{F}(t)}\,dx, \quad t \ge 0.}
}
\examples{
# Exponential(1)
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
residual_entropy(f, Fb, t = 0)   # = 1
residual_entropy(f, Fb, t = 1)   # = 1 + exp(-1) * log(exp(1)-1) or similar

}
\references{
Ebrahimi N., Pellerey F. (1995). New partial ordering of survival functions
based on the notion of uncertainty. *Journal of Applied Probability*, 32(1),
202--211.
}
\seealso{
\code{\link{residual_info_gen}}, \code{\link{shannon_entropy}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_reversed.R
\name{blt_reversed}
\alias{blt_reversed}
\title{Bivariate Laplace Transform of Reversed Residual Lives}
\usage{
blt_reversed(s1, s2, t1, t2, cdf_fn = NULL, theta = 0, g_form = FALSE)
}
\arguments{
\item{s1, s2}{Positive Laplace parameters.}

\item{t1, t2}{Positive truncation times (\eqn{t_i > 0}).}

\item{cdf_fn}{A function \code{function(x1, x2)} returning the joint CDF
\eqn{F(x_1,x_2)}.  Defaults to the FGM distribution.}

\item{theta}{FGM association parameter (used when \code{cdf_fn = NULL}).}

\item{g_form}{Logical; if \code{TRUE} returns the \eqn{G} form instead of
\eqn{L}.}
}
\value{
A named numeric vector \code{(L1, L2)} or \code{(G1, G2)}.
}
\description{
Computes the bivariate Laplace transform of reversed (past) residual lives.
For a random pair \eqn{(X_1,X_2)} with joint distribution function
\eqn{F(x_1,x_2)}, the transform is
\deqn{L_{t_1|t_2}(s_1) = e^{-s_1 t_1} +
  \frac{s_1 \int_0^{t_1} e^{-s_1 x_1} F(x_1, t_2)\,dx_1}{F(t_1, t_2)}}
and the associated \eqn{G} function (useful for ordering) is
\deqn{G_1(t_1,t_2) =
  \frac{\int_0^{t_1} e^{-s_1 x_1} F(x_1, t_2)\,dx_1}{
    e^{-s_1 t_1} F(t_1,t_2)}.}
}
\examples{
# FGM distribution (default)
blt_reversed(s1 = 1, s2 = 1, t1 = 0.6, t2 = 0.6)

# G form
blt_reversed(s1 = 1, s2 = 1, t1 = 0.6, t2 = 0.6, g_form = TRUE)

# User-supplied CDF
cdf <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
blt_reversed(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5, cdf_fn = cdf)

}
\references{
Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*.
}
\seealso{
\code{\link{blt_reversed_fgm}}, \code{\link{blt_order_reversed}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{blt_residual_gumbel}
\alias{blt_residual_gumbel}
\title{Closed-Form Bivariate Laplace Transform of Residual Lives (Gumbel)}
\usage{
blt_residual_gumbel(s1, s2, t1 = 0, t2 = 0, k1 = 1, k2 = 1, theta = 0)
}
\arguments{
\item{s1, s2}{Positive Laplace parameters.}

\item{t1, t2}{Non-negative truncation ages.}

\item{k1, k2}{Positive rate parameters.}

\item{theta}{Non-negative association parameter.}
}
\value{
A named numeric vector \code{(L1_star, L2_star)}.
}
\description{
Returns the *star* bivariate Laplace transform of residual lives for the
Gumbel bivariate exponential distribution in closed form:
\deqn{L^*_{X_{t_1|t_2}}(s_1) = \frac{1}{k_1 + s_1 + \theta t_2},\quad
  L^*_{X_{t_2|t_1}}(s_2) = \frac{1}{k_2 + s_2 + \theta t_1}.}
}
\examples{
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5)
blt_residual_gumbel(s1 = 2, s2 = 0.5, t1 = 0, t2 = 1, k1 = 2, k2 = 1, theta = 0.3)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022), Example 3.1.
\doi{10.1080/03610926.2022.2085874}
}
\seealso{
\code{\link{blt_residual}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{regf_characterise}
\alias{regf_characterise}
\title{Characterise a Distribution via the REGF}
\usage{
regf_characterise(
  dens_fn,
  surv_fn = NULL,
  alpha = 2,
  t_grid = seq(0, 2, by = 0.2),
  upper = 100
)
}
\arguments{
\item{dens_fn}{Density function.}

\item{surv_fn}{Survival function.}

\item{alpha}{REGF parameter.}

\item{t_grid}{Grid of truncation times.}

\item{upper}{Integration upper bound.}
}
\value{
A data frame with columns \code{t}, \code{REGF}, \code{log_REGF},
  and the fitted linear-in-\eqn{t} slope \code{slope} (log-linear
  characterisation).
}
\description{
Checks whether the functional form of \eqn{\mathcal{I}_\alpha(X;t)} over a
grid of \eqn{t} values is consistent with an exponential distribution.  For
\eqn{X \sim \text{Exp}(\lambda)},
\eqn{\mathcal{I}_\alpha(t) / \mathcal{I}_\alpha(0) = e^{(\alpha-1)\lambda t}},
a pure exponential in \eqn{t}.
}
\examples{
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
regf_characterise(f, Fb, alpha = 2)

}
\references{
Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 3.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{biv_whr_order}
\alias{biv_whr_order}
\title{Weak Bivariate Hazard Rate Order}
\usage{
biv_whr_order(
  surv_X,
  surv_Y,
  t1_grid = seq(0.1, 3, by = 0.5),
  t2_grid = seq(0.1, 3, by = 0.5),
  tol = 1e-06
)
}
\arguments{
\item{surv_X, surv_Y}{Joint survival functions.}

\item{t1_grid, t2_grid}{Evaluation grids.}

\item{tol}{Tolerance.}
}
\value{
A list: \code{order_holds} (logical), \code{violations} (data frame).
}
\description{
Checks whether \eqn{X \le_{\rm whr} Y}: the ratio
\eqn{\bar{F}_Y(x_1,x_2) / \bar{F}_X(x_1,x_2)} is increasing in
\eqn{(x_1,x_2)}, i.e.\ \eqn{h_{i,X}(t_1,t_2) \ge h_{i,Y}(t_1,t_2),\; i=1,2}.
}
\examples{
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0)
biv_whr_order(sX, sY)

}
\references{
Shaked M., Shanthikumar J.G. (2007). *Stochastic Orders*. Springer.
Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_regf}
\alias{plot_regf}
\title{Plot Residual Entropy Generating Function}
\usage{
plot_regf(
  dens_fn,
  surv_fn = NULL,
  alpha = 2,
  t = 0,
  grid = NULL,
  vary = c("t", "alpha"),
  upper = 100,
  col = "purple",
  lwd = 2,
  main = NULL
)
}
\arguments{
\item{dens_fn}{Density function.}

\item{surv_fn}{Survival function; computed if \code{NULL}.}

\item{alpha}{Positive parameter (used when \code{vary = "t"}).}

\item{t}{Truncation time (used when \code{vary = "alpha"}).}

\item{grid}{Numeric vector of the varying quantity values.}

\item{vary}{One of \code{"t"} (plot as function of \eqn{t}) or
\code{"alpha"} (plot as function of \eqn{\alpha}).}

\item{upper}{Integration upper bound.}

\item{col}{Line colour.}

\item{lwd}{Line width.}

\item{main}{Plot title.}
}
\value{
Invisibly returns the data frame used for plotting.
}
\description{
Plots the REGF \eqn{\mathcal{I}_\alpha(X;t)} as a function of \eqn{t} for
given \eqn{\alpha}, or as a function of \eqn{\alpha} for given \eqn{t}.
}
\examples{
f  <- function(x) dexp(x, 1)
Fb <- function(x) pexp(x, 1, lower.tail = FALSE)
plot_regf(f, Fb, alpha = 2, vary = "t")
plot_regf(f, Fb, t = 0.5, vary = "alpha")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{biv_wrhr_order}
\alias{biv_wrhr_order}
\title{Weak Bivariate Reversed Hazard Rate Order}
\usage{
biv_wrhr_order(
  cdf_X,
  cdf_Y,
  x1_grid = seq(0.1, 0.9, by = 0.2),
  x2_grid = seq(0.1, 0.9, by = 0.2),
  tol = 1e-06
)
}
\arguments{
\item{cdf_X, cdf_Y}{Joint CDFs.}

\item{x1_grid, x2_grid}{Evaluation grids (positive values).}

\item{tol}{Tolerance.}
}
\value{
List with \code{order_holds} and \code{violations}.
}
\description{
Checks \eqn{X \le_{\rm wrhr} Y}: the ratio
\eqn{F_X(x_1,x_2)/F_Y(x_1,x_2)} is decreasing in \eqn{(x_1,x_2)}, i.e.\
\eqn{h_{i,X}(x_1,x_2) \le h_{i,Y}(x_1,x_2),\; i=1,2}.
}
\examples{
cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.2)
cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
biv_wrhr_order(cX, cY)

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{bivpower}
\alias{bivpower}
\alias{pbivpower}
\alias{sbivpower}
\alias{dbivpower}
\alias{rbivpower}
\title{Bivariate Power Distribution}
\usage{
pbivpower(x1, x2, p1 = 1, p2 = 1, theta = 0)

sbivpower(x1, x2, p1 = 1, p2 = 1, theta = 0)

dbivpower(x1, x2, p1 = 1, p2 = 1, theta = 0)

rbivpower(n, p1 = 1, p2 = 1, theta = 0)
}
\arguments{
\item{x1, x2}{Values in the support.}

\item{p1, p2}{Positive shape parameters.}

\item{theta}{Association parameter, \eqn{0 \le \theta \le 1}.}

\item{n}{Number of random observations.}
}
\value{
Numeric vector or two-column matrix (\code{rbivpower}).
}
\description{
Distribution function, survival function, density, and random generation for
the bivariate power distribution:
\deqn{F(x_1,x_2) = x_1^{p_1 + \theta \log x_2}\,
  x_2^{p_2},\quad 0 \le x_1,x_2 \le p_2,\; p_1,p_2 > 0,\; 0 \le \theta \le 1.}
}
\examples{
pbivpower(0.4, 0.5, p1 = 2, p2 = 2, theta = 0.3)
set.seed(7); head(rbivpower(30, p1 = 2, p2 = 2, theta = 0.2))

}
\references{
Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{blt_order_residual}
\alias{blt_order_residual}
\title{Bivariate Laplace Transform Order of Residual Lives (BLt-rl)}
\usage{
blt_order_residual(
  surv_X,
  surv_Y,
  s1 = 1,
  s2 = 1,
  t1_grid = seq(0.1, 3, by = 0.5),
  t2_grid = seq(0.1, 3, by = 0.5),
  tol = 1e-06
)
}
\arguments{
\item{surv_X, surv_Y}{Joint survival functions for \eqn{X} and \eqn{Y}
respectively, each of the form \code{function(x1, x2)}.}

\item{s1, s2}{Laplace parameters to evaluate at.}

\item{t1_grid, t2_grid}{Grids of truncation times (default 0.1 to 3).}

\item{tol}{Tolerance for declaring inequality (default \code{1e-6}).}
}
\value{
A list with elements:
  \describe{
    \item{\code{order_holds}}{Logical; \code{TRUE} if \eqn{X \le Y} at all
      grid points.}
    \item{\code{violations}}{Data frame of grid points where the ordering
      fails.}
    \item{\code{ratio_matrix}}{Matrix of \eqn{L^*_X / L^*_Y} values.}
  }
}
\description{
Checks whether random vector \eqn{X} is smaller than \eqn{Y} in the
bivariate Laplace transform order of residual lives (BLt-rl).

\eqn{X \le_{\rm BLt\text{-}rl} Y} if and only if for all
\eqn{(t_1,t_2)} in the support:
\deqn{\frac{\int_{t_1}^\infty e^{-s_1 x_1}\bar{F}_X(x_1,t_2)\,dx_1}{
  e^{-s_1 t_1}\bar{F}_X(t_1,t_2)}
  \ge
  \frac{\int_{t_1}^\infty e^{-s_1 x_1}\bar{F}_Y(x_1,t_2)\,dx_1}{
  e^{-s_1 t_1}\bar{F}_Y(t_1,t_2)}}
i.e.\ \eqn{L^*_{X_{t_1|t_2}}(s_1) \ge L^*_{Y_{t_1|t_2}}(s_1)} for all
\eqn{t_1,t_2,s_1}.  The function evaluates this inequality at a grid.
}
\examples{
# Compare two Gumbel distributions
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1, theta = 0.2)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 2, theta = 0.2)
blt_order_residual(sX, sY, s1 = 1, s2 = 1,
                   t1_grid = c(0.1, 0.5, 1), t2_grid = c(0.1, 0.5))

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022), Definition 4.1 & Prop. 4.1.
\doi{10.1080/03610926.2022.2085874}
}
\seealso{
\code{\link{blt_residual}}, \code{\link{biv_whr_order}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{nbuhr_test}
\alias{nbuhr_test}
\title{Test NBUHR / NWUHR Aging Class}
\usage{
nbuhr_test(
  t2 = 1,
  t1_grid = seq(0.1, 5, by = 0.1),
  surv_fn = NULL,
  k1 = 1,
  k2 = 1,
  theta = 0
)
}
\arguments{
\item{t2}{Fixed value of the second age coordinate.}

\item{t1_grid}{Numeric vector of \eqn{t_1} values to check (default 0.1 to
5 in steps of 0.1).}

\item{surv_fn}{Joint survival function; defaults to Gumbel bivariate
exponential.}

\item{k1, k2, theta}{Gumbel parameters.}
}
\value{
A list with components:
  \describe{
    \item{\code{class1}}{Character: \code{"NBUHR"}, \code{"NWUHR"}, or
      \code{"neither"} for the first component.}
    \item{\code{class2}}{Same for the second component.}
    \item{\code{h1_grid}}{Numeric vector of \eqn{h_1(t_1,t_2)} values.}
    \item{\code{h2_grid}}{Numeric vector of \eqn{h_2(t_1,t_2)} values.}
  }
}
\description{
Checks whether a bivariate lifetime distribution belongs to the NBUHR
(New Better than Used in Hazard Rate) or NWUHR (New Worse than Used) aging
class.  A distribution is NBUHR if
\deqn{h_1(0,t_2) \ge h_1(t_1,t_2)\;\text{ for all }t_1 > 0}
and similarly for the second component.  The function evaluates this at a
grid of \eqn{t_1} values.
}
\examples{
nbuhr_test(t2 = 1, k1 = 1, k2 = 1, theta = 0.3)
nbuhr_test(t2 = 0.5,
           surv_fn = function(x1, x2) exp(-(x1 + x2)))

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022), Definition 3.2.
\doi{10.1080/03610926.2022.2085874}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{sim_blt_residual}
\alias{sim_blt_residual}
\title{Monte-Carlo Simulation Study for the BLT Residual Estimator}
\usage{
sim_blt_residual(
  n_obs = 200,
  n_sim = 100,
  s1 = 1,
  s2 = 1,
  t1 = 0.3,
  t2 = 0.3,
  k1 = 1,
  k2 = 1,
  theta = 0.5,
  seed = 42L
)
}
\arguments{
\item{n_obs}{Sample size per replicate.}

\item{n_sim}{Number of simulation replicates.}

\item{s1, s2}{Laplace parameters.}

\item{t1, t2}{Truncation ages.}

\item{k1, k2, theta}{Gumbel parameters.}

\item{seed}{Random seed for reproducibility.}
}
\value{
A data frame with columns \code{component}, \code{true_value},
  \code{mean_est}, \code{bias}, \code{variance}, \code{mse}.
}
\description{
Evaluates the performance of \code{\link{np_blt_residual}} via repeated
simulation from the Gumbel bivariate exponential distribution and compares
estimates to the closed-form \code{\link{blt_residual_gumbel}} values.
Returns bias, variance, and mean squared error (MSE).
}
\examples{
sim_blt_residual(n_obs = 100, n_sim = 50, s1 = 1, s2 = 1,
                 t1 = 0.3, t2 = 0.3, k1 = 1, k2 = 1, theta = 0.5)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022), Section 6.
\doi{10.1080/03610926.2022.2085874}
}
\seealso{
\code{\link{np_blt_residual}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{fgm_biv}
\alias{fgm_biv}
\alias{pfgm_biv}
\alias{dfgm_biv}
\alias{sfgm_biv}
\alias{rfgm_biv}
\title{Farlie-Gumbel-Morgenstern (FGM) Bivariate Distribution}
\usage{
pfgm_biv(x1, x2, theta = 0)

dfgm_biv(x1, x2, theta = 0)

sfgm_biv(x1, x2, theta = 0)

rfgm_biv(n, theta = 0)
}
\arguments{
\item{x1, x2}{Values in \eqn{[0,1]}.}

\item{theta}{Association parameter, \eqn{|\theta| \le 1}.}

\item{n}{Number of random observations.}
}
\value{
Numeric vector (scalar functions) or two-column matrix
  (\code{rfgm_biv}).
}
\description{
Density, distribution function, survival function, and random generation for
the FGM bivariate distribution on \eqn{[0,1]^2}:
\deqn{F(x_1, x_2) = x_1 x_2 \bigl[1 + \theta(1-x_1)(1-x_2)\bigr],
  \quad 0 \le x_1, x_2 \le 1,\; |\theta| \le 1.}
}
\examples{
pfgm_biv(0.4, 0.6, theta = 0.5)
dfgm_biv(0.4, 0.6, theta = 0.5)
set.seed(1); head(rfgm_biv(50, theta = 0.5))

}
\references{
Jayalekshmi S., Rajesh G. Bivariate Laplace transform order and ordering of
reversed residual lives. *International Journal of Reliability, Quality and
Safety Engineering*.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_entropy.R
\name{sim_regf}
\alias{sim_regf}
\title{Monte-Carlo Simulation for the REGF Nonparametric Estimator}
\usage{
sim_regf(lambda = 1, alpha = 2, t = 0, n_obs = 200, n_sim = 100, seed = 42L)
}
\arguments{
\item{lambda}{Exponential rate.}

\item{alpha}{REGF parameter.}

\item{t}{Truncation time.}

\item{n_obs}{Sample size per replicate.}

\item{n_sim}{Number of replicates.}

\item{seed}{Random seed.}
}
\value{
A data frame with \code{true_value}, \code{mean_est}, \code{bias},
  \code{variance}, \code{mse}.
}
\description{
Evaluates the performance of \code{\link{np_residual_info_gen}} via repeated
simulation from the exponential distribution and compares to the true REGF
\eqn{\mathcal{I}_\alpha(\text{Exp}(\lambda); t) =
  \lambda^{\alpha-1} e^{(\alpha-1)\lambda t} / (\alpha-1)}.
}
\examples{
sim_regf(lambda = 1, alpha = 2, t = 0, n_obs = 100, n_sim = 30)

}
\references{
Smitha S., Rajesh G., Jayalekshmi S. (2024), Section 4.
}
\seealso{
\code{\link{np_residual_info_gen}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{blt_residual}
\alias{blt_residual}
\title{Bivariate Laplace Transform of Residual Lives}
\usage{
blt_residual(
  s1,
  s2,
  t1 = 0,
  t2 = 0,
  surv_fn = NULL,
  k1 = 1,
  k2 = 1,
  theta = 0,
  upper = 50,
  star = FALSE
)
}
\arguments{
\item{s1, s2}{Positive Laplace parameters.}

\item{t1, t2}{Non-negative time thresholds (truncation ages).}

\item{surv_fn}{A function \code{function(x1, x2)} returning the joint
survival probability \eqn{\bar{F}(x_1, x_2)}.  Defaults to the
Gumbel bivariate exponential with \code{k1}, \code{k2}, \code{theta}.}

\item{k1, k2}{Rate parameters (used only when \code{surv_fn = NULL}).}

\item{theta}{Association parameter (used only when \code{surv_fn = NULL}).}

\item{upper}{Integration upper bound (default \code{50}).}

\item{star}{Logical; if \code{TRUE} returns the star version \eqn{L^*}.}
}
\value{
A named numeric vector with elements \code{L1} and \code{L2}
  (or \code{L1_star} and \code{L2_star} when \code{star = TRUE}).
}
\description{
Computes the bivariate Laplace transform of residual lives
\eqn{L_{X_{t_1|t_2}}(s_1)} and \eqn{L_{X_{t_2|t_1}}(s_2)} defined as
\deqn{L_{X_{t_1|t_2}}(s_1)
  = \frac{\int_{t_1}^{\infty} e^{-s_1 x_1}
    f(x_1 | X_2 > t_2)\,dx_1}{e^{-s_1 t_1} \bar{F}(t_1 | X_2 > t_2)}}
and analogously for the second component.

The *star* version (used for ordering) is
\deqn{L^*_{X_{t_1|t_2}}(s_1) = \frac{1 - L_{X_{t_1|t_2}}(s_1)}{s_1}
  = \frac{\int_{t_1}^{\infty} e^{-s_1 x_1}
    \bar{F}(x_1, t_2)}{e^{-s_1 t_1}\bar{F}(t_1,t_2)}\,dx_1.}
}
\examples{
# Gumbel bivariate exponential, default parameters
blt_residual(s1 = 1, s2 = 1, t1 = 0.5, t2 = 0.5)

# Star version used in ordering
blt_residual(s1 = 0.5, s2 = 0.5, t1 = 1, t2 = 1, star = TRUE)

# User-supplied survival function (Schur-constant exponential)
surv <- function(x1, x2) exp(-(x1 + x2))
blt_residual(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, surv_fn = surv)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022).
\doi{10.1080/03610926.2022.2085874}
}
\seealso{
\code{\link{blt_residual_gumbel}}, \code{\link{blt_order_residual}},
  \code{\link{np_blt_residual}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{biv_brlmr_order}
\alias{biv_brlmr_order}
\title{Bivariate Relative Mean Residual Life Order}
\usage{
biv_brlmr_order(
  surv_X,
  surv_Y,
  t2_fixed = 0.5,
  t1_grid = seq(0.2, 3, by = 0.3),
  tol = 1e-06
)
}
\arguments{
\item{surv_X, surv_Y}{Joint survival functions.}

\item{t2_fixed}{Fixed value of \eqn{t_2} for the univariate slices.}

\item{t1_grid}{Grid of \eqn{t_1} values.}

\item{tol}{Tolerance.}
}
\value{
List with \code{order_holds} and \code{ratio_values}.
}
\description{
Checks whether \eqn{X \le_{\rm brlmr} Y}: the ratio
\eqn{m_{i,Y}(t_1,t_2) / m_{i,X}(t_1,t_2)} is increasing in \eqn{t_i}.
}
\examples{
sX <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 2, k2 = 1)
sY <- function(x1, x2) sgumbel_biv(x1, x2, k1 = 1, k2 = 1)
biv_brlmr_order(sX, sY, t2_fixed = 0.5)

}
\references{
Kayid M., Izadkhah S., Alshami S. (2016). Residual probability function,
associated orderings, and related aging classes. *Statistics and Probability
Letters*, 116, 37--47.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{biv_wrmrl_order}
\alias{biv_wrmrl_order}
\title{Weak Bivariate Reversed Mean Residual Life Order}
\usage{
biv_wrmrl_order(
  cdf_X,
  cdf_Y,
  x1_grid = seq(0.2, 0.8, by = 0.2),
  x2_grid = seq(0.2, 0.8, by = 0.2),
  tol = 1e-06
)
}
\arguments{
\item{cdf_X, cdf_Y}{Joint CDFs.}

\item{x1_grid, x2_grid}{Evaluation grids.}

\item{tol}{Tolerance.}
}
\value{
List with \code{order_holds} and \code{violations}.
}
\description{
Checks \eqn{X \le_{\rm wrmrl} Y}:
\eqn{m_{i,X}(x_1,x_2) \ge m_{i,Y}(x_1,x_2),\; i=1,2}.
}
\examples{
cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.1)
cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.8)
biv_wrmrl_order(cX, cY)

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Section 2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_blt_reversed}
\alias{plot_blt_reversed}
\title{Plot Bivariate Laplace Transform of Reversed Residual Lives}
\usage{
plot_blt_reversed(
  cdf_fn,
  cdf_fn2 = NULL,
  s1 = 1,
  t2 = 0.5,
  t1_grid = seq(0.1, 0.9, by = 0.05),
  theta = 0,
  xlab = expression(t[1]),
  ylab = expression(L[t[1] * "|" * t[2]](s[1])),
  main = "Bivariate LT of Reversed Residual Lives",
  col1 = "darkgreen",
  col2 = "darkorange",
  lwd = 2,
  legend_labels = c("Distribution 1", "Distribution 2")
)
}
\arguments{
\item{cdf_fn}{Joint CDF function.}

\item{cdf_fn2}{Optional second CDF for comparison.}

\item{s1}{Laplace parameter.}

\item{t2}{Fixed second truncation time.}

\item{t1_grid}{Grid of first truncation times.}

\item{theta}{FGM parameter (used if \code{cdf_fn = NULL}).}

\item{xlab, ylab, main}{Plot labels.}

\item{col1, col2}{Line colours.}

\item{lwd}{Line width.}

\item{legend_labels}{Legend labels.}
}
\value{
Invisibly returns the data frame used for plotting.
}
\description{
Plots the reversed-life Laplace transform \eqn{L_{t_1|t_2}(s_1)} as a
function of \eqn{t_1} for fixed \eqn{s_1} and \eqn{t_2}.
}
\examples{
cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.2)
cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.7)
plot_blt_reversed(cX, cY, s1 = 1, t2 = 0.5,
                  legend_labels = c("theta=0.2", "theta=0.7"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bivariate_laplace_residual.R
\name{np_blt_residual}
\alias{np_blt_residual}
\title{Nonparametric Estimator for the Bivariate Laplace Transform of Residual Lives}
\usage{
np_blt_residual(data, s1, s2, t1 = 0, t2 = 0)
}
\arguments{
\item{data}{A two-column numeric matrix or data frame with columns for
\eqn{X_1} and \eqn{X_2}.}

\item{s1, s2}{Positive Laplace parameters.}

\item{t1, t2}{Non-negative truncation ages.}
}
\value{
A named numeric vector \code{(L1_hat, L2_hat)}.
}
\description{
Given a bivariate sample \eqn{(X_{1i}, X_{2i}),\; i=1,\ldots,n}, estimates
\deqn{\hat{L}^*_{1}(s_1; t_1, t_2)
  = \frac{\sum_{i:X_{1i}>t_1,\,X_{2i}>t_2}
    \int_{t_1}^{X_{1i}} e^{-s_1 u}\,du}{e^{-s_1 t_1} \cdot
    \#\{X_{1i}>t_1,X_{2i}>t_2\}}}
and analogously for the second component, using the empirical survival
function as described in Jayalekshmi et al. (2022), Section 6.
}
\examples{
set.seed(123)
dat <- rgumbel_biv(200, k1 = 1, k2 = 1, theta = 0.5)
np_blt_residual(dat, s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3)

# Compare with closed form
blt_residual_gumbel(s1 = 1, s2 = 1, t1 = 0.3, t2 = 0.3, theta = 0.5)

}
\references{
Jayalekshmi S., Rajesh G., Nair N.U. (2022), Section 6.
\doi{10.1080/03610926.2022.2085874}
}
\seealso{
\code{\link{blt_residual_gumbel}}, \code{\link{sim_blt_residual}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_orders.R
\name{blt_order_reversed}
\alias{blt_order_reversed}
\title{Bivariate Laplace Transform Order of Reversed Residual Lives (BLt-Rrl)}
\usage{
blt_order_reversed(
  cdf_X,
  cdf_Y,
  s1 = 1,
  s2 = 1,
  t1_grid = seq(0.2, 0.8, by = 0.2),
  t2_grid = seq(0.2, 0.8, by = 0.2),
  tol = 1e-06
)
}
\arguments{
\item{cdf_X, cdf_Y}{Joint CDF functions for \eqn{X} and \eqn{Y}.}

\item{s1, s2}{Laplace parameters.}

\item{t1_grid, t2_grid}{Grids of truncation times.}

\item{tol}{Tolerance.}
}
\value{
Same structure as \code{\link{blt_order_residual}}.
}
\description{
Checks whether \eqn{X \le_{\rm BLt\text{-}Rrl} Y}: for all
\eqn{(t_1,t_2)}, \eqn{L^{X}_{t_1|t_2}(s_1) \ge L^{Y}_{t_1|t_2}(s_2)}.
}
\examples{
cX <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.3)
cY <- function(x1, x2) pfgm_biv(x1, x2, theta = 0.5)
blt_order_reversed(cX, cY, s1 = 1, s2 = 1,
                   t1_grid = c(0.2, 0.5), t2_grid = c(0.2, 0.5))

}
\references{
Jayalekshmi S., Rajesh G. — IJRQSE, Definition 2.
}
\seealso{
\code{\link{blt_reversed}}, \code{\link{biv_wrhr_order}}
}
